@@ -4,7 +4,7 @@
  * GChemPaint library
  * atom.cc
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -37,9 +37,10 @@
 #include <canvas/gcp-canvas-bpath.h>
 #include <canvas/gcp-canvas-pango.h>
 #include <gcu/element.h>
+#include <goffice/math/go-math.h>
 #include <openbabel/mol.h>
 #include <glib/gi18n-lib.h>
-#include <cmath>
+#include <cstdlib>
 #include <cstring>
 
 using namespace gcu;
@@ -70,7 +71,7 @@ Atom::Atom (): gcu::Atom (),
 	m_ChargeAutoPos = true;
 	m_Layout = m_ChargeLayout = NULL;
 	m_DrawCircle = false;
-	m_InkRect.width = m_HInkRect.width = 0.;
+	m_SWidth = 0.;
 }
 
 Atom::~Atom ()
@@ -112,7 +113,7 @@ Atom::Atom (int Z, double x, double y, double z): gcu::Atom (Z, x, y, z),
 	m_ChargeAutoPos = true;
 	m_Layout = m_ChargeLayout = NULL;
 	m_DrawCircle = false;
-	m_InkRect.width = m_HInkRect.width = 0.;
+	m_SWidth = 0.;
 }
 
 Atom::Atom (OBAtom* atom): gcu::Atom (),
@@ -140,19 +141,21 @@ Atom::Atom (OBAtom* atom): gcu::Atom (),
 	m_Layout = m_ChargeLayout = NULL;
 	m_DrawCircle = false;
 	m_Charge = atom->GetFormalCharge ();
-	m_InkRect.width = m_HInkRect.width = 0.;
+	m_SWidth = 0.;
 }
 
 void Atom::SetZ (int Z)
 {
 	gcu::Atom::SetZ (Z);
+	if (Z < 1)
+		return;
 	m_Element = Element::GetElement (m_Z);
-	if ((m_Valence = m_Element->GetDefaultValence ()))
+	if ((m_Valence = (m_Element)? m_Element->GetDefaultValence (): 0))
 		m_HPos = (m_HPosStyle == AUTO_HPOS)? GetBestSide(): m_HPosStyle;
 	else
 		m_nH = 0;
-	int max = m_Element->GetMaxValenceElectrons ();
-	int diff = m_Element->GetTotalValenceElectrons () - m_Element->GetValenceElectrons ();
+	int max = (m_Element)? m_Element->GetMaxValenceElectrons (): 0;
+	int diff = (m_Element)? m_Element->GetTotalValenceElectrons () - m_Element->GetValenceElectrons (): 0;
 	switch (max) {
 	case 2:
 		m_ValenceOrbitals = 1;
@@ -181,9 +184,9 @@ void Atom::SetZ (int Z)
 	EmitSignal (OnChangedSignal);
 }
 
-int Atom::GetTotalBondsNumber ()
+int Atom::GetTotalBondsNumber () const
 {
-	std::map<gcu::Atom*, gcu::Bond*>::iterator i, end = m_Bonds.end ();
+	std::map<gcu::Atom*, gcu::Bond*>::const_iterator i, end = m_Bonds.end ();
 	int n = 0;
 	for (i = m_Bonds.begin(); i != end; i++)
 		n += (*i).second->GetOrder ();
@@ -238,7 +241,7 @@ void Atom::Update ()
 		electron = (Electron*) GetNextChild (i);
 	}
 	int nbonds = GetTotalBondsNumber ();
-	if (m_Valence > 0) {
+	if (m_Valence > 0 && !m_Element->IsMetallic ()) {
 		m_nlp = (m_Element->GetValenceElectrons () - m_Valence) / 2;
 		if ((m_Charge > 0) && (m_nlp > 0)) m_nlp -= (m_Charge + 1) / 2;
 		else if (m_Charge < 0)
@@ -270,14 +273,14 @@ void Atom::Update ()
 	} else {
 		m_nH = 0;
 		if (m_ChargeAuto || !m_Charge) {
-			m_Charge = m_Element->GetValenceElectrons () - 2 * nexplp - nexplu - nbonds;
+			m_Charge = (m_Element)? m_Element->GetValenceElectrons () - 2 * nexplp - nexplu - nbonds: 0;
 			if (m_Charge > 0)
 				m_Charge = 0;
 			m_ChargeAuto = true;
 		}
 	}
 	Document *pDoc = (Document *) GetDocument ();
-	if (pDoc)
+	if (pDoc && pDoc->GetView ())
 		m_Changed = pDoc->GetView ()->GetNbWidgets ();
 	m_AvailPosCached = false;
 	if (nbonds && GetZ () == 6) {
@@ -301,12 +304,14 @@ void Atom::Update ()
 			double angle = static_cast<Bond*> ((*i).second)->GetAngle2D (this);
 			i++;
 			angle -= static_cast<Bond*> ((*i).second)->GetAngle2D (this);
-			while (angle < 0)
-				angle += 360.;
-			while (angle > 360.)
-				angle -= 360;
-			if (fabs (angle - 180.) < 1)
-				DrawCircle = true;
+			if (go_finite (angle)) {
+				while (angle < 0)
+					angle += 360.;
+				while (angle > 360.)
+					angle -= 360;
+				if (fabs (angle - 180.) < 1)
+					DrawCircle = true;
+			}
 		}
 		if (DrawCircle != m_DrawCircle) {
 			m_DrawCircle = DrawCircle;
@@ -314,22 +319,13 @@ void Atom::Update ()
 		}
 	}
 }
-	
-bool Atom::IsInCycle (Cycle* pCycle)
-{
-	map<gcu::Atom*, gcu::Bond*>::iterator i, end = m_Bonds.end ();
-	for (i = m_Bonds.begin (); i != end; i++)
-		if (((Bond*) (*i).second)->IsInCycle (pCycle))
-		return true;
-	return false;
-}
 
-void Atom::Add (GtkWidget* w)
+void Atom::Add (GtkWidget* w) const
 {
-	if (!w)
+	if (!w || !GetZ ())
 		return;
 	if (m_Changed > 0)
-		m_Changed--;
+		const_cast <Atom *> (this)->m_Changed--;
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	if (pData->Items[this] != NULL)
 		return;
@@ -337,25 +333,25 @@ void Atom::Add (GtkWidget* w)
 	Theme *pTheme = pView->GetDoc ()->GetTheme ();
 	if (m_Layout == NULL) {
 		PangoContext* pc = pView->GetPangoContext ();
-		m_Layout = pango_layout_new (pc);
+		const_cast <Atom *> (this)->m_Layout = pango_layout_new (pc);
 	}
 	if (m_FontName != pView->GetFontName ()) {
 		pango_layout_set_font_description (m_Layout, pView->GetPangoFontDesc ());
 		pango_layout_set_text (m_Layout, "l", 1);
 		PangoLayoutIter* iter = pango_layout_get_iter (m_Layout);
-		m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+		const_cast <Atom *> (this)->m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
 		pango_layout_iter_free (iter);
-		m_FontName = pView->GetFontName ();
-		m_CHeight = 0.;
+		const_cast <Atom *> (this)->m_FontName = pView->GetFontName ();
+		const_cast <Atom *> (this)->m_CHeight = 0.;
 	}
 	PangoRectangle rect;
 	if (m_CHeight == 0.) {
 		pango_layout_set_text (m_Layout, "C", 1);
 		pango_layout_get_extents (m_Layout, &rect, NULL);
-		m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
+		const_cast <Atom *> (this)->m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
 	}
 	double x, y, xc = 0., yc;
-	m_width =  m_height = 2.0 * pTheme->GetPadding ();
+	const_cast <Atom *> (this)->m_width = const_cast <Atom *> (this)->m_height = 2.0 * pTheme->GetPadding ();
 	GetCoords (&x, &y);
 	x *= pTheme->GetZoomFactor ();
 	y *= pTheme->GetZoomFactor ();
@@ -363,14 +359,20 @@ void Atom::Add (GtkWidget* w)
 	GnomeCanvasGroup *group, *chgp;
 	group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL));
 	g_signal_connect (G_OBJECT (group), "event", G_CALLBACK (on_event), w);
-	g_object_set_data (G_OBJECT (group), "object", this);
+	g_object_set_data (G_OBJECT (group), "object", (void *) this);
 	if ((GetZ () != 6) || (GetBondsNumber () == 0)) {
 		int sw, sp;
 		const gchar* symbol = GetSymbol (), *text;
 		sw = strlen (symbol);
 		pango_layout_set_text (m_Layout, symbol, sw);
 		pango_layout_get_extents (m_Layout, &rect, NULL);
-		m_width += rect.width / PANGO_SCALE;
+		const_cast <Atom *> (this)->m_width += rect.width / PANGO_SCALE;
+		const_cast <Atom *> (this)->BuildSymbolGeometry ((double) rect.width / PANGO_SCALE, (double) rect.height / PANGO_SCALE, m_ascent - (double) rect.y / PANGO_SCALE - m_CHeight);
+/*			m_SWidth = (double) rect.width / PANGO_SCALE / 2.;
+		const_cast <Atom *> (this)->m_SHeightH = m_ascent - (double) rect.y / PANGO_SCALE - m_CHeight;
+		const_cast <Atom *> (this)->m_SHeightL = (double) rect.height / PANGO_SCALE - m_SHeightH;
+		const_cast <Atom *> (this)->m_SAngleH = atan2 (m_SHeightH, m_SWidth);
+		const_cast <Atom *> (this)->m_SAngleL = atan2 (m_SHeightL, m_SWidth);*/
 		int n = GetAttachedHydrogens ();
 		if (n > 0) {
 			if (n > 1) {
@@ -413,12 +415,12 @@ void Atom::Add (GtkWidget* w)
 			pango_layout_set_text (m_Layout, text, -1);
 		}
 		pango_layout_get_extents (m_Layout, NULL, &rect);
-		m_length = double (rect.width / PANGO_SCALE);
-		m_text_height = m_height = rect.height / PANGO_SCALE;
+		const_cast <Atom *> (this)->m_length = double (rect.width / PANGO_SCALE);
+		const_cast <Atom *> (this)->m_text_height = const_cast <Atom *> (this)->m_height = rect.height / PANGO_SCALE;
 		pango_layout_index_to_pos (m_Layout, sp, &rect);
 		int st = rect.x / PANGO_SCALE;
 		pango_layout_index_to_pos (m_Layout, sp + sw, &rect);
-		m_lbearing = (st + rect.x / PANGO_SCALE) / 2.;
+		const_cast <Atom *> (this)->m_lbearing = (st + rect.x / PANGO_SCALE) / 2.;
 
 		item = gnome_canvas_item_new (
 							group,
@@ -427,11 +429,12 @@ void Atom::Add (GtkWidget* w)
 							"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding (),
 							"x2", x - m_lbearing + m_length + pTheme->GetPadding (),
 							"y2", y  - m_ascent + m_CHeight + m_height + pTheme->GetPadding (),
-							"fill_color", (pData->IsSelected (this))? SelectColor: "white",
+							"fill_color", (pData->IsSelected (this))? SelectColor: NULL,
 							NULL);
 		g_object_set_data (G_OBJECT (group), "rect", item);
 		g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
-		g_object_set_data (G_OBJECT (item), "object", this);
+		g_object_set_data (G_OBJECT (item), "object", (void *) this);
+		gnome_canvas_item_hide (item);
 		
 		item = gnome_canvas_item_new (
 							group,
@@ -441,7 +444,7 @@ void Atom::Add (GtkWidget* w)
 							"layout", m_Layout,
 						NULL);
 		g_object_set_data (G_OBJECT (group), "symbol", item);
-		g_object_set_data (G_OBJECT (item), "object", this);
+		g_object_set_data (G_OBJECT (item), "object", (void *) this);
 		g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
 	} else {
 		item = gnome_canvas_item_new (
@@ -451,14 +454,14 @@ void Atom::Add (GtkWidget* w)
 								"y1", y - 3,
 								"x2", x + 3,
 								"y2", y + 3,
-								"fill_color",  (pData->IsSelected (this))? SelectColor: "white",
+								"fill_color",  (pData->IsSelected (this))? SelectColor: NULL,
 								NULL);
 		g_object_set_data(G_OBJECT (group), "rect", item);
 		gnome_canvas_request_redraw ((GnomeCanvas*) w, (int) x - 3, (int) y - 3, (int) x + 3, (int) y + 3);
 		gnome_canvas_item_lower_to_bottom (GNOME_CANVAS_ITEM (group));
 		gnome_canvas_item_raise (GNOME_CANVAS_ITEM (group), 1);
 		g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
-		g_object_set_data (G_OBJECT (item), "object", this);
+		g_object_set_data (G_OBJECT (item), "object", (void *) this);
 		if (m_DrawCircle) {
 			double dx = pTheme->GetStereoBondWidth () / 2.;
 			item = gnome_canvas_item_new (
@@ -472,16 +475,16 @@ void Atom::Add (GtkWidget* w)
 									NULL);
 			g_object_set_data (G_OBJECT (group), "bullet", item);
 			g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
-			g_object_set_data (G_OBJECT (item), "object", this);
+			g_object_set_data (G_OBJECT (item), "object", (void *) this);
 		}
 	}
 	pData->Items[this] = group;
-	m_width /= pTheme->GetZoomFactor ();
-	m_height /= pTheme->GetZoomFactor ();
+	const_cast <Atom *> (this)->m_width /= pTheme->GetZoomFactor ();
+	const_cast <Atom *> (this)->m_height /= pTheme->GetZoomFactor ();
 	/* add charge */
 	int charge = GetCharge ();
 	if (charge) {
-		int align = GetChargePosition (m_ChargePos, m_ChargeAngle * 180 / M_PI, x, y);
+		int align = const_cast <Atom *> (this)->GetChargePosition (const_cast <Atom *> (this)->m_ChargePos, m_ChargeAngle * 180 / M_PI, x, y);
 		if (m_ChargeDist != 0.) {
 			align = 0;
 			x = m_x + m_ChargeDist * cos (m_ChargeAngle);
@@ -495,16 +498,16 @@ void Atom::Add (GtkWidget* w)
 			PangoRectangle rect;
 			if (!m_ChargeLayout) {
 				PangoContext* pc = pData->m_View->GetPangoContext();
-				m_ChargeLayout = pango_layout_new (pc);
+				const_cast <Atom *> (this)->m_ChargeLayout = pango_layout_new (pc);
 				pango_layout_set_font_description (m_ChargeLayout, pData->m_View->GetPangoSmallFontDesc ());
 			}
 			pango_layout_set_text (m_ChargeLayout, fig, -1);
 			pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
-			m_ChargeWidth = rect.width / PANGO_SCALE;
-			m_ChargeTWidth = m_ChargeWidth + 1. + pTheme->GetChargeSignSize ();
+			const_cast <Atom *> (this)->m_ChargeWidth = rect.width / PANGO_SCALE;
+			const_cast <Atom *> (this)->m_ChargeTWidth = m_ChargeWidth + 1. + pTheme->GetChargeSignSize ();
 		} else {
-			m_ChargeWidth = 0.;
-			m_ChargeTWidth = pTheme->GetChargeSignSize ();
+			const_cast <Atom *> (this)->m_ChargeWidth = 0.;
+			const_cast <Atom *> (this)->m_ChargeTWidth = pTheme->GetChargeSignSize ();
 		}
 		switch (align) {
 		case -2:
@@ -585,19 +588,21 @@ void Atom::Add (GtkWidget* w)
 		gnome_canvas_path_def_unref (cpd);
 		g_object_set_data (G_OBJECT (group), "sign", item);
 	}
-	map<string, Object*>::iterator i;
-	Object* electron = GetFirstChild (i);
+	map<string, Object*>::const_iterator i;
+	Object const *electron = GetFirstChild (i);
 	while (electron){
 		electron->Add (w);
 		electron = GetNextChild (i);
 	}
 }
 
-void Atom::Update (GtkWidget* w)
+void Atom::Update (GtkWidget* w) const
 {
-	if (!w)
+	if (!w || !GetZ ())
 		return;
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	if (pData->Items[this] == NULL)
+		return;
 	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
 	double x, y, xc = 0., yc;
 	GetCoords (&x, &y);
@@ -612,17 +617,17 @@ void Atom::Update (GtkWidget* w)
 		pango_layout_set_font_description (m_Layout, pView->GetPangoFontDesc ());
 		pango_layout_set_text (Layout, "l", 1);
 		PangoLayoutIter* iter = pango_layout_get_iter (Layout);
-		m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+		const_cast <Atom *> (this)->m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
 		pango_layout_iter_free (iter);
-		m_FontName = pView->GetFontName ();
+		const_cast <Atom *> (this)->m_FontName = pView->GetFontName ();
 		pango_layout_set_text (Layout, "C", 1);
 		PangoRectangle rect;
 		pango_layout_get_extents (Layout, &rect, NULL);
-		m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
+		const_cast <Atom *> (this)->m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
 		g_object_unref (G_OBJECT (Layout));
 	}
 	if (m_Changed)
-		BuildItems (pData);
+		const_cast <Atom *> (this)->BuildItems (pData);
 	else {
 		if ((GetZ() != 6) || (GetBondsNumber () == 0) || m_ShowSymbol) {
 			g_object_set (G_OBJECT (g_object_get_data (G_OBJECT (group), "symbol")),
@@ -657,7 +662,7 @@ void Atom::Update (GtkWidget* w)
 	int charge = GetCharge ();
 	if (charge) {
 		if (item) {
-			int align = GetChargePosition (m_ChargePos, m_ChargeAngle * 180. / M_PI, x, y);
+			int align = const_cast <Atom *> (this)->GetChargePosition (const_cast <Atom *> (this)->m_ChargePos, m_ChargeAngle * 180. / M_PI, x, y);
 			if (m_ChargeDist != 0.) {
 				align = 0;
 				x = m_x + m_ChargeDist * cos (m_ChargeAngle);
@@ -672,15 +677,15 @@ void Atom::Update (GtkWidget* w)
 				PangoRectangle rect;
 				if (!m_ChargeLayout) {
 					PangoContext* pc = pData->m_View->GetPangoContext ();
-					m_ChargeLayout = pango_layout_new (pc);
+					const_cast <Atom *> (this)->m_ChargeLayout = pango_layout_new (pc);
 					pango_layout_set_font_description (m_ChargeLayout, pData->m_View->GetPangoSmallFontDesc ());
 				}
 				pango_layout_set_text (m_ChargeLayout, fig, -1);
 				pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
-				m_ChargeWidth = rect.width / PANGO_SCALE;
+				const_cast <Atom *> (this)->m_ChargeWidth = rect.width / PANGO_SCALE;
 			} else
-				m_ChargeWidth = 0.;
-			m_ChargeTWidth = m_ChargeWidth + 1. + pTheme->GetChargeSignSize ();
+				const_cast <Atom *> (this)->m_ChargeWidth = 0.;
+			const_cast <Atom *> (this)->m_ChargeTWidth = m_ChargeWidth + 1. + pTheme->GetChargeSignSize ();
 			if (figure == NULL && fig != NULL) {
 				figure = gnome_canvas_item_new (
 							GNOME_CANVAS_GROUP (item),
@@ -753,7 +758,7 @@ void Atom::Update (GtkWidget* w)
 			gnome_canvas_path_def_unref (cpd);
 		} else {
 			GnomeCanvasGroup *chgp;
-			int align = GetChargePosition(m_ChargePos, m_ChargeAngle * 180 / M_PI, x, y);
+			int align = const_cast <Atom *> (this)->GetChargePosition(const_cast <Atom *> (this)->m_ChargePos, m_ChargeAngle * 180 / M_PI, x, y);
 			if (m_ChargeDist != 0.) {
 				align = 0;
 				x = (m_x + m_ChargeDist * cos (m_ChargeAngle));
@@ -767,17 +772,17 @@ void Atom::Update (GtkWidget* w)
 				fig = g_strdup_printf ("%d", abs (charge));
 				if (!m_ChargeLayout) {
 					PangoContext* pc = pData->m_View->GetPangoContext ();
-					m_ChargeLayout = pango_layout_new (pc);
+					const_cast <Atom *> (this)->m_ChargeLayout = pango_layout_new (pc);
 					pango_layout_set_font_description (m_ChargeLayout, pData->m_View->GetPangoSmallFontDesc ());
 				}
 				pango_layout_set_text (m_ChargeLayout, fig, -1);
 				PangoRectangle rect;
 				pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
-				m_ChargeWidth = rect.width / PANGO_SCALE;
-				m_ChargeTWidth = m_ChargeWidth + pTheme->GetPadding () + pTheme->GetChargeSignSize ();
+				const_cast <Atom *> (this)->m_ChargeWidth = rect.width / PANGO_SCALE;
+				const_cast <Atom *> (this)->m_ChargeTWidth = m_ChargeWidth + pTheme->GetPadding () + pTheme->GetChargeSignSize ();
 			} else {
-				m_ChargeWidth = 0.;
-				m_ChargeTWidth = pTheme->GetChargeSignSize ();
+				const_cast <Atom *> (this)->m_ChargeWidth = 0.;
+				const_cast <Atom *> (this)->m_ChargeTWidth = pTheme->GetChargeSignSize ();
 			}
 			switch (align) {
 			case -2:
@@ -866,8 +871,8 @@ void Atom::Update (GtkWidget* w)
 			g_object_set_data ((GObject*)group, "circle", NULL);
 			g_object_set_data ((GObject*)group, "sign", NULL);
 	}
-	map<string, Object*>::iterator i;
-	Object* electron = GetFirstChild (i);
+	map<string, Object*>::const_iterator i;
+	Object const *electron = GetFirstChild (i);
 	while (electron){
 		electron->Update (w);
 		electron = GetNextChild (i);
@@ -1160,11 +1165,13 @@ void Atom::SetSelected (GtkWidget* w, int state)
 	GnomeCanvasGroup* group = pData->Items[this];
 	gpointer item;
 	gchar const *color, *chargecolor;
+	bool visible = true;
 	
 	switch (state) {	
 	case SelStateUnselected:
-		color = "white";
+		color = NULL;
 		chargecolor = "black";
+			visible = false;
 		break;
 	case SelStateSelected:
 		chargecolor = color = SelectColor;
@@ -1176,12 +1183,17 @@ void Atom::SetSelected (GtkWidget* w, int state)
 		chargecolor = color = DeleteColor;
 		break;
 	default:
-		color = "white";
+		color = NULL;
 		chargecolor = "black";
 		break;
 	}
-	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
+	item = g_object_get_data (G_OBJECT (group), "rect");
+	g_object_set (G_OBJECT (item),
 				"fill_color", color, NULL);
+	if (visible)
+		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
+	else
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
 	if ((item = g_object_get_data (G_OBJECT (group), "bullet")))
 		g_object_set (item, "fill_color", chargecolor, NULL);
 	if ((item = g_object_get_data (G_OBJECT (group), "figure")))
@@ -1472,12 +1484,12 @@ void Atom::NotifyPositionOccupation (unsigned char pos, bool occupied)
 		m_OccupiedPos &= ~pos;
 }
 	
-xmlNodePtr Atom::Save (xmlDocPtr xml)
+xmlNodePtr Atom::Save (xmlDocPtr xml) const
 {
 	xmlNodePtr node = gcu::Atom::Save (xml), child;
 	if (node) {
 	// Save electrons
-		map<string, Object*>::iterator i;
+		map<string, Object*>::const_iterator i;
 		Electron* electron = (Electron*) GetFirstChild (i);
 		while (electron){ 
 			child = electron->Save (xml);
@@ -1656,7 +1668,7 @@ void Atom::SetChargePosition (unsigned char Pos, bool def, double angle, double
 	m_AvailPosCached = false;
 }
 
-char Atom::GetChargePosition (double *Angle, double *Dist)
+char Atom::GetChargePosition (double *Angle, double *Dist) const
 {
 	if (Angle)
 		*Angle = m_ChargeAngle;
@@ -1757,4 +1769,36 @@ bool Atom::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double
 	return result | Object::BuildContextualMenu (UIManager, object, x, y);
 }
 
+bool Atom::Match (gcu::Atom *atom, AtomMatchState &state)
+{
+	if (m_nH != static_cast <Atom*> (atom)->m_nH)
+		return false;
+	return gcu::Atom::Match (atom, state);
+}
+
+void Atom::GetSymbolGeometry (double &width, double &height, double &angle, bool up) const
+{
+	if ((GetZ() != 6) || (GetBondsNumber () == 0) || m_ShowSymbol) {
+		width = m_SWidth;
+		if (up) {
+			height = m_SHeightH ;
+			angle = m_SAngleH;
+		} else {
+			height = m_SHeightL ;
+			angle = m_SAngleL;
+		}
+	} else
+		width = height  = angle = 0.;
+}
+
+void Atom::BuildSymbolGeometry (double width, double height, double ascent)
+{
+	m_SWidth = width / 2.;
+	m_SHeightH = ascent;
+	m_SHeightL = height - m_SHeightH;
+	m_SAngleH = atan2 (m_SHeightH, m_SWidth);
+	m_SAngleL = atan2 (m_SHeightL, m_SWidth);
+}
+
+
 }	//	namespace gcp
