@@ -4,7 +4,7 @@
  * GChemPaint library
  * bond.cc 
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,7 +26,6 @@
 #include "bond.h"
 #include "atom.h"
 #include "fragment.h"
-#include "cycle.h"
 #include "settings.h"
 #include "document.h"
 #include "theme.h"
@@ -36,6 +35,8 @@
 #include <canvas/gcp-canvas-line.h>
 #include <canvas/gcp-canvas-bpath.h>
 #include <canvas/gcp-canvas-polygon.h>
+#include <gcu/cycle.h>
+#include <gcu/objprops.h>
 #include <glib/gi18n-lib.h>
 #include <cmath>
 #include <cstring>
@@ -89,49 +90,6 @@ double Bond::GetAngle2D (Atom* pAtom)
 	return HUGE_VAL;
 }
 
-double Bond::GetAngle2DRad (Atom* pAtom)
-{
-	double x1, y1, x2, y2;
-	m_Begin->GetCoords (&x1, &y1);
-	m_End->GetCoords (&x2, &y2);
-	x2 -= x1;
-	y2 -= y1;
-	double length = square (x2) + square (y2);
-	if (length == 0.0)
-		return HUGE_VAL;
-	if (pAtom == m_Begin)
-		return atan2 (-y2, x2);
-	else if (pAtom == m_End)
-		return atan2 (y2, -x2);
-	return HUGE_VAL;
-}
-
-Cycle* Bond::GetFirstCycle (std::list<Cycle*>::iterator& i, Cycle * pCycle)
-{
-	i = m_Cycles.begin ();
-	return GetNextCycle (i, pCycle);
-}
-
-Cycle* Bond::GetNextCycle (std::list<Cycle*>::iterator& i, Cycle * pCycle)
-{
-	if (*i == pCycle)
-		i++;
-	if (i == m_Cycles.end ())
-		return NULL;
-	pCycle = *i;
-	i++;
-	return pCycle;
-}
-
-bool Bond::IsInCycle (Cycle* pCycle)
-{
-	std::list<Cycle*>::iterator i, end = m_Cycles.end ();
-	for (i = m_Cycles.begin (); i != end; i++)
-		if ((*i) == pCycle)
-			return true;
-	return false;
-}
-
 bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, double* y2)
 {
 	if ((Num == 0) || (Num > m_order))
@@ -145,6 +103,43 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 		double BondDist = Theme->GetBondDist () / Theme->GetZoomFactor ();
 		dx *= (BondDist / l);
 		dy *= (BondDist / l);
+		// now, exclude symbols rectangles from the drawing
+		double ax, ay, anga, angb = atan2 (fabs (dy), fabs (dx));
+		reinterpret_cast <Atom *> (m_Begin)->GetSymbolGeometry (ax, ay, anga, dy < 0);
+		bool horizontal;
+		if (ax > 0) {
+			horizontal = anga >= angb;
+			if (horizontal) {
+				ax = (ax + 2.) / Theme->GetZoomFactor ();
+				if (dx > 0)
+					ax = - ax;
+				*x1 -= ax;
+				*y1 -= ax * dy / dx;
+			} else {
+				ay = (ay + 2.) / Theme->GetZoomFactor ();
+				if (dy > 0)
+					ay = - ay;
+				*y1 -= ay;
+				*x1 -= ay * dx / dy;
+			}
+		}
+		reinterpret_cast <Atom *> (m_End)->GetSymbolGeometry (ax, ay, anga, dy > 0);
+		if (ax > 0) {
+			horizontal = anga >= angb;
+			if (horizontal) {
+				ax = (ax + 2.) / Theme->GetZoomFactor ();
+				if (dx > 0)
+					ax = - ax;
+				*x2 += ax;
+				*y2 += ax * dy / dx;
+			} else {
+				ay = (ay + 2.) / Theme->GetZoomFactor ();
+				if (dy > 0)
+					ay = - ay;
+				*y2 += ay;
+				*x2 += ay * dx / dy;
+			}
+		}
 		if (m_order & 1) {
 			m_coords[0] = *x1;
 			m_coords[1] = *y1;
@@ -168,7 +163,7 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 			Cycle* pCycle;
 			if (IsCyclic() > 1) {
 				//Search prefered cycle
-				std::list<Cycle*>::iterator i = m_Cycles.begin (), end = m_Cycles.end ();
+				list<Cycle*>::iterator i = m_Cycles.begin (), end = m_Cycles.end ();
 				pCycle = *i;
 				for (; i != end; i++)
 					if (pCycle->IsBetterForBonds (*i))
@@ -238,7 +233,7 @@ Object* Bond::GetAtomAt(double x, double y, double z)
 	return NULL;
 }
 
-bool Bond::SaveNode (xmlDocPtr xml, xmlNodePtr node)
+bool Bond::SaveNode (xmlDocPtr xml, xmlNodePtr node) const
 {
 	switch(m_type) {
 	case UpBondType:
@@ -292,6 +287,8 @@ bool Bond::LoadNode (xmlNodePtr node)
 
 void Bond::IncOrder (int n)
 {
+	if (!m_Begin || !m_End)
+		return;
 	if (!((Atom*) GetAtom(0))->AcceptNewBonds () ||
 		!((Atom*) GetAtom (1))->AcceptNewBonds ())
 		m_order = 1;
@@ -308,6 +305,8 @@ void Bond::IncOrder (int n)
 double Bond::GetDist (double x, double y)
 {
 	double x1, y1, x2, y2, l, d1, d2;
+	if (!m_Begin || !m_End)
+		return G_MAXDOUBLE;
 	Theme *Theme = dynamic_cast <Document *> (GetDocument ())->GetTheme ();
 	double BondDist = Theme->GetBondDist () / Theme->GetZoomFactor ();
 	m_Begin->GetCoords (&x1, &y1);
@@ -328,14 +327,14 @@ double Bond::GetDist (double x, double y)
 
 void Bond::AddCycle (Cycle* pCycle)
 {
-	m_Cycles.push_back (pCycle);
+	gcu::Bond::AddCycle (pCycle);
 	if ((m_order == 2) && m_CoordsCalc)
 		SetDirty();
 }
 
 void Bond::RemoveCycle (Cycle* pCycle)
 {
-	m_Cycles.remove (pCycle);
+	gcu::Bond::RemoveCycle (pCycle);
 	if ((m_order == 2) && m_CoordsCalc)
 		SetDirty();
 }
@@ -350,7 +349,7 @@ void Bond::SetDirty ()
 
 void Bond::RemoveAllCycles ()
 {
-	m_Cycles.clear ();
+	gcu::Bond::RemoveAllCycles ();
 	if (m_order == 2) {
 		Document *pDoc = (Document*) GetDocument ();
 		if (pDoc)
@@ -377,20 +376,14 @@ void Bond::Revert ()
 	m_End = pAtom;
 }
 
-double Bond::Get2DLength ()
-{
-	double x1, y1, x2, y2;
-	m_Begin->GetCoords (&x1, &y1);
-	m_End->GetCoords (&x2, &y2);
-	return sqrt (square (x1 - x2) + square (y1 - y2));
-}
-
 void Bond::SetSelected (GtkWidget* w, int state)
 {
 	if (!m_order)
 		return;
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	GnomeCanvasGroup* group = pData->Items[this];
+	if (!group)
+		return; // this should not occur, but it might crash if it does occur after loading a bad file.
 	gchar const *color;
 	switch (state) {	
 	case SelStateUnselected:
@@ -424,7 +417,7 @@ void Bond::SetSelected (GtkWidget* w, int state)
 	}
 }
 
-void Bond::Add (GtkWidget* w)
+void Bond::Add (GtkWidget* w) const
 {
 	if (!w)
 		return;
@@ -434,7 +427,7 @@ void Bond::Add (GtkWidget* w)
 	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
 	gcu::Atom *pAtom0, *pAtom1;
 	if (!(pAtom0 = GetAtom (0)))
-		return; 
+		return;
 	if (!(pAtom1 = GetAtom (1)))
 		return;
 	unsigned char order = GetOrder ();
@@ -442,20 +435,19 @@ void Bond::Add (GtkWidget* w)
 		return;
 	GnomeCanvasGroup* group;
 	GnomeCanvasItem* item = NULL;
-	pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL));
 	g_signal_connect (G_OBJECT (group), "event", G_CALLBACK (on_event), w);
-	g_object_set_data (G_OBJECT (group), "object", this);
+	g_object_set_data (G_OBJECT (group), "object", (void *) this);
 	bool result = false;
 	if (m_Crossing.size () > 0) {
-		map<Bond*, BondCrossing>::iterator i, iend = m_Crossing.end ();
+		map<Bond*, BondCrossing>::const_iterator i, iend = m_Crossing.end ();
 		for (i = m_Crossing.begin (); i != iend; i++)
 			if ((result |= (*i).second.is_before))
 				break;
 	}
 	GnomeCanvasPathDef *path = NULL;
 	if (result) {
-		path = BuildCrossingPathDef (pData);
+		path = const_cast <Bond *> (this)->BuildCrossingPathDef (pData);
 		if (path) {
 			GnomeCanvasItem *item = (m_type == NormalBondType || m_type == UndeterminedBondType)?
 				gnome_canvas_item_new (
@@ -473,7 +465,7 @@ void Bond::Add (GtkWidget* w)
 					"width_units", 0.,
 					NULL);
 			g_object_set_data (G_OBJECT (group), "back", item);
-			g_object_set_data (G_OBJECT (item), "object", this);
+			g_object_set_data (G_OBJECT (item), "object", (void *) this);
 			g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
 			/* now bring to front, FIXME: not secure ! */
 			gnome_canvas_item_lower_to_bottom (item);
@@ -487,7 +479,7 @@ void Bond::Add (GtkWidget* w)
 			gnome_canvas_path_def_unref (path);
 		}
 	}
-	path = BuildPathDef (pData);
+	path = const_cast <Bond *> (this)->BuildPathDef (pData);
 	switch (GetType ())
 	{
 		case NormalBondType:
@@ -514,7 +506,7 @@ void Bond::Add (GtkWidget* w)
 	}
 	gnome_canvas_path_def_unref (path);
 	g_object_set_data (G_OBJECT (group), "path", item);
-	g_object_set_data (G_OBJECT (item), "object", this);
+	g_object_set_data (G_OBJECT (item), "object", (void *) this);
 	g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
 	pData->Items[this] = group;
 	if (pAtom0->GetParent ()->GetType () == FragmentType)
@@ -542,24 +534,6 @@ void Bond::Add (GtkWidget* w)
 	}
 }
 
-bool Bond::ReplaceAtom (Atom* oldAtom, Atom* newAtom)
-{
-	if (oldAtom == m_Begin) {
-		if (m_End)
-			m_End->RemoveBond (this);
-		m_Begin = newAtom;
-		if (m_Begin && m_End)
-			m_End->AddBond (this);
-	} else if (oldAtom == m_End) {
-		if (m_Begin)
-			m_Begin->RemoveBond (this);
-		m_End = newAtom;
-		if (m_Begin && m_End)
-			m_Begin->AddBond (this);
-	}
-	return true;
-}
-
 GnomeCanvasPathDef* Bond::BuildPathDef (WidgetData* pData)
 {
 	double x1, y1, x2, y2, x, y, dx, dy, dx1, dy1;
@@ -725,15 +699,17 @@ GnomeCanvasPathDef *Bond::BuildCrossingPathDef (WidgetData* pData)
 	return path;
 }
 
-void Bond::Update(GtkWidget* w)
+void Bond::Update(GtkWidget* w) const
 {
 	if (!w || !m_order)
 		return;
 	WidgetData* pData = (WidgetData*) g_object_get_data(G_OBJECT(w), "data");
+	if (pData->Items[this] == NULL)
+		return;
 	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
 	bool result = false;
 	if (m_Crossing.size () > 0) {
-		map<Bond*, BondCrossing>::iterator i, iend = m_Crossing.end ();
+		map<Bond*, BondCrossing>::const_iterator i, iend = m_Crossing.end ();
 		for (i = m_Crossing.begin (); i != iend; i++)
 			if ((result |= (*i).second.is_before))
 				break;
@@ -741,9 +717,13 @@ void Bond::Update(GtkWidget* w)
 	GnomeCanvasPathDef* path;
 	void * obj;
 	GnomeCanvasGroup *group = pData->Items[this];
+	if (!group) {
+		Add (w);
+		return;
+	}
 	obj = g_object_get_data(G_OBJECT(group), "back");
 	if (result) {
-		path = BuildCrossingPathDef (pData);
+		path = const_cast <Bond *> (this)->BuildCrossingPathDef (pData);
 		if (path) {
 			if (obj)
 				g_object_set (obj, "bpath", path, NULL);
@@ -764,7 +744,7 @@ void Bond::Update(GtkWidget* w)
 						"width_units", 0.,
 						NULL);
 				g_object_set_data (G_OBJECT (group), "back", item);
-				g_object_set_data (G_OBJECT (item), "object", this);
+				g_object_set_data (G_OBJECT (item), "object", (void *) this);
 				g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
 				/* now bring to front, FIXME: not secure ! */
 				gnome_canvas_item_lower_to_bottom (item);
@@ -781,7 +761,7 @@ void Bond::Update(GtkWidget* w)
 	} else if (obj) {
 		g_object_set_data (G_OBJECT(group), "back", NULL);
 	}
-	path = BuildPathDef (pData);
+	path = const_cast <Bond *> (this)->BuildPathDef (pData);
 	obj = g_object_get_data(G_OBJECT(group), "path");
 	g_object_set (obj, "bpath", path, NULL);
 	if (m_type == NormalBondType || m_type == UndeterminedBondType)
@@ -930,4 +910,44 @@ void Bond::BringToFront ()
 	pView->Update (this);
 }
 
+struct BondTypeStruct {
+	BondType type;
+	bool invert;
+};
+static map<string, struct BondTypeStruct> BondTypesValues;
+bool Bond::SetProperty (unsigned property, char const *value)
+{
+	switch (property) {
+	case GCU_PROP_BOND_TYPE: {
+		if (BondTypesValues.size () == 0) {
+			struct BondTypeStruct s;
+			s.invert = false;
+			s.type = NormalBondType;
+			BondTypesValues["normal"] = s;
+			s.type = ForeBondType;
+			BondTypesValues["bold"] = s;
+			s.type = UpBondType;
+			BondTypesValues["wedge"] = s;
+			s.type = DownBondType;
+			BondTypesValues["hash"] = s;
+			s.invert = true;
+			s.type = UpBondType;
+			BondTypesValues["wedge-invert"] = s;
+			s.type = DownBondType;
+			BondTypesValues["hash-invert"] = s;
+		}
+		map<string, struct BondTypeStruct>::iterator it;
+		if ((it = BondTypesValues.find (value)) != BondTypesValues.end ()) {
+			m_type = (*it).second.type;
+			if ((*it).second.invert)
+				Revert ();
+		}
+		break;
+	}
+	default:
+		gcu::Bond::SetProperty (property, value);
+	}
+	return  true;
+}
+
 }	//	namespace gcp
