@@ -38,6 +38,7 @@
 #include <canvas/gcp-canvas-rect-ellipse.h>
 #include <canvas/gcp-canvas-bpath.h>
 #include <gcu/element.h>
+#include <gcu/objprops.h>
 #include <pango/pango-attributes.h>
 #include <glib/gi18n-lib.h>
 #include <list>
@@ -59,7 +60,9 @@ static void on_fragment_sel_changed (Fragment *fragment, struct GnomeCanvasPango
 	fragment->OnSelChanged (bounds);
 }
 
-Fragment::Fragment (): TextObject (FragmentType)
+Fragment::Fragment ():
+	TextObject (FragmentType),
+	m_Inversable (false)
 {
 	m_Atom = new FragmentAtom (this, 0);
 	m_BeginAtom = m_EndAtom = 0;
@@ -69,7 +72,9 @@ Fragment::Fragment (): TextObject (FragmentType)
 	SetId ("f1");
 }
 
-Fragment::Fragment (double x, double y): TextObject (x, y, FragmentType)
+Fragment::Fragment (double x, double y):
+	TextObject (x, y, FragmentType),
+	m_Inversable (false)
 {
 	m_Atom = new FragmentAtom (this, 0);
 	m_Atom->SetCoords (x, y);
@@ -240,7 +245,7 @@ bool Fragment::OnChanged (bool save)
 	return true;
 }
 
-void Fragment::Add (GtkWidget* w)
+void Fragment::Add (GtkWidget* w) const
 {
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	if (pData->Items[this] != NULL)
@@ -249,38 +254,38 @@ void Fragment::Add (GtkWidget* w)
 	Theme *pTheme = pView->GetDoc ()->GetTheme ();
 	if (m_ascent <= 0) {
 		PangoContext* pc = pView->GetPangoContext ();
-		m_Layout = pango_layout_new (pc);
+		const_cast <Fragment *> (this)->m_Layout = pango_layout_new (pc);
 		PangoAttrList *l = pango_attr_list_new ();
 		pango_layout_set_attributes (m_Layout, l);
 		pango_layout_set_font_description (m_Layout, pView->GetPangoFontDesc ());
 		pango_layout_set_text (m_Layout, "l", 1);
 		PangoLayoutIter* iter = pango_layout_get_iter (m_Layout);
-		m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+		const_cast <Fragment *> (this)->m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
 		pango_layout_iter_free (iter);
 		pango_layout_set_text (m_Layout, "C", 1);
 		PangoRectangle rect;
 		pango_layout_get_extents (m_Layout, &rect, NULL);
-		m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
+		const_cast <Fragment *> (this)->m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
 		pango_layout_set_text (m_Layout, m_buf.c_str (), -1);
 		if (m_AttrList) {
 			pango_layout_set_attributes (m_Layout, m_AttrList);
 			pango_attr_list_unref (m_AttrList);
-			m_AttrList = NULL;
+			const_cast <Fragment *> (this)->m_AttrList = NULL;
 		}
 		if (m_buf.length () > 0) {
-			m_buf.clear ();
+			const_cast <Fragment *> (this)->m_buf.clear ();
 			pango_layout_index_to_pos (m_Layout, m_BeginAtom, &rect);
-			m_lbearing = rect.x / PANGO_SCALE;
+			const_cast <Fragment *> (this)->m_lbearing = rect.x / PANGO_SCALE;
 			pango_layout_index_to_pos (m_Layout, m_EndAtom, &rect);
-			m_lbearing += rect.x / PANGO_SCALE;
-			m_lbearing /=  2;
+			const_cast <Fragment *> (this)->m_lbearing += rect.x / PANGO_SCALE;
+			const_cast <Fragment *> (this)->m_lbearing /=  2;
 			iter = pango_layout_get_iter (m_Layout);
-			m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+			const_cast <Fragment *> (this)->m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
 			pango_layout_iter_free (iter);
 		}
 		pango_layout_get_extents (m_Layout, NULL, &rect);
-		m_length = rect.width / PANGO_SCALE;
-		m_height = rect.height / PANGO_SCALE;
+		const_cast <Fragment *> (this)->m_length = rect.width / PANGO_SCALE;
+		const_cast <Fragment *> (this)->m_height = rect.height / PANGO_SCALE;
 	}
 	GnomeCanvasGroup* group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL)), *chgp;
 	GnomeCanvasItem* item = gnome_canvas_item_new(
@@ -290,12 +295,11 @@ void Fragment::Add (GtkWidget* w)
 						"y1", m_y * pTheme->GetZoomFactor () - pTheme->GetPadding () - m_ascent + m_CHeight,
 						"x2", m_x * pTheme->GetZoomFactor () + m_length + pTheme->GetPadding () - m_lbearing,
 						"y2", m_y * pTheme->GetZoomFactor () + m_height + pTheme->GetPadding () - m_ascent + m_CHeight,
-						"fill_color", "white",
-						"outline_color", "white",
 						NULL);
+	gnome_canvas_item_hide (item);
 	g_object_set_data (G_OBJECT (group), "rect", item);
 	g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
-	g_object_set_data (G_OBJECT (item), "object", this);
+	g_object_set_data (G_OBJECT (item), "object", (void *) this);
 	item = gnome_canvas_item_new (
 						group,
 						gnome_canvas_pango_get_type (),
@@ -305,16 +309,16 @@ void Fragment::Add (GtkWidget* w)
 						"editing", false,
 						NULL);
 	g_object_set_data (G_OBJECT (group), "fragment", item);
-	g_object_set_data (G_OBJECT (item), "object", this);
+	g_object_set_data (G_OBJECT (item), "object", (void *) this);
 	g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
-	g_signal_connect_swapped (G_OBJECT (item), "changed", G_CALLBACK (on_fragment_changed), this);
-	g_signal_connect_swapped (G_OBJECT (item), "sel-changed", G_CALLBACK (on_fragment_sel_changed), this);
+	g_signal_connect_swapped (G_OBJECT (item), "changed", G_CALLBACK (on_fragment_changed), (void *) this);
+	g_signal_connect_swapped (G_OBJECT (item), "sel-changed", G_CALLBACK (on_fragment_sel_changed), (void *) this);
 	/* add charge */
 	int charge = m_Atom->GetCharge ();
 	if (charge) {
 		double x, y, Angle, Dist;
 		unsigned char Pos = m_Atom->Atom::GetChargePosition (&Angle, &Dist);
-		int align = GetChargePosition(m_Atom, Pos, 0., x, y);
+		int align = const_cast <Fragment *> (this)->GetChargePosition (m_Atom, Pos, 0., x, y);
 		if (Dist != 0.) {
 			x = m_x + Dist * cos (Angle);
 			y = m_y - Dist * sin (Angle);
@@ -387,6 +391,7 @@ void Fragment::Add (GtkWidget* w)
 		g_object_set_data (G_OBJECT (group), "sign", item);
 	}
 	pData->Items[this] = group;
+	const_cast <FragmentAtom *> (m_Atom)->DoBuildSymbolGeometry (pData->m_View);
 }
 
 void Fragment::SetSelected (GtkWidget* w, int state)
@@ -394,10 +399,12 @@ void Fragment::SetSelected (GtkWidget* w, int state)
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	GnomeCanvasGroup* group = pData->Items[this];
 	gchar const *chargecolor, *color;
+	bool visible = true;
 	switch (state) {	
 	case SelStateUnselected:
-		color = "white";
+		color = NULL;
 		chargecolor = "black";
+		visible = false;
 		break;
 	case SelStateSelected:
 		chargecolor = color = SelectColor;
@@ -409,20 +416,25 @@ void Fragment::SetSelected (GtkWidget* w, int state)
 		chargecolor = color = DeleteColor;
 		break;
 	default:
-		color = "white";
+		color = NULL;
 		chargecolor = "black";
 		break;
 	}
-	g_object_set (G_OBJECT (g_object_get_data (G_OBJECT (group), "rect")),
-				"fill_color", color, NULL);
 	gpointer item;
+	item = g_object_get_data (G_OBJECT (group), "rect");
+	g_object_set (G_OBJECT (item),
+				"fill_color", color, NULL);
+	if (visible)
+		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
+	else
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
 	if ((item = g_object_get_data (G_OBJECT (group), "circle")))
 		g_object_set (item, "outline_color", chargecolor, NULL);
 	if ((item = g_object_get_data (G_OBJECT (group), "sign")))
 		g_object_set (item, "outline_color", chargecolor, NULL);
 }
 
-void Fragment::Update (GtkWidget* w)
+void Fragment::Update (GtkWidget* w) const
 {
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
@@ -445,7 +457,7 @@ void Fragment::Update (GtkWidget* w)
 		double x, y, Angle, Dist;
 		unsigned char Pos = m_Atom->Atom::GetChargePosition (&Angle, &Dist);
 		if (item) {
-			int align = GetChargePosition (m_Atom, Pos, Angle, x, y);
+			int align = const_cast <Fragment *> (this)->GetChargePosition (m_Atom, Pos, Angle, x, y);
 			if (Dist != 0.) {
 				x = m_x + Dist * cos (Angle);
 				y = m_y - Dist * sin (Angle);
@@ -504,7 +516,7 @@ void Fragment::Update (GtkWidget* w)
 			gnome_canvas_path_def_unref (cpd);
 		} else {
 			GnomeCanvasGroup *chgp;
-			int align = GetChargePosition (m_Atom, Pos, Angle, x, y);
+			int align = const_cast <Fragment *> (this)->GetChargePosition (m_Atom, Pos, Angle, x, y);
 			x *= pTheme->GetZoomFactor ();
 			if (Dist != 0.) {
 				x = m_x + Dist * cos (Angle);
@@ -582,12 +594,13 @@ void Fragment::Update (GtkWidget* w)
 			g_object_set_data ((GObject*) group, "charge", NULL);
 		}
 	}
+	const_cast <FragmentAtom *> (m_Atom)->DoBuildSymbolGeometry (pData->m_View);
 }
 
-xmlNodePtr Fragment::Save (xmlDocPtr xml)
+xmlNodePtr Fragment::Save (xmlDocPtr xml) const
 {
-	m_buf = pango_layout_get_text (m_Layout);
-	if (m_RealSave && !Validate ())
+	const_cast <Fragment *> (this)->m_buf = pango_layout_get_text (m_Layout);
+	if (m_RealSave && !const_cast <Fragment *> (this)->Validate ())
 		return NULL;
 	xmlNodePtr node = xmlNewDocNode (xml, NULL, (xmlChar*) "fragment", NULL);
 	if (m_buf.length ()) {
@@ -633,7 +646,7 @@ bool filter_func (PangoAttribute *attribute, struct FilterStruct *s)
 	return FALSE;
 }
 
-bool Fragment::SavePortion (xmlDocPtr xml, xmlNodePtr node, unsigned start, unsigned end)
+bool Fragment::SavePortion (xmlDocPtr xml, xmlNodePtr node, unsigned start, unsigned end) const
 {
 	xmlNodePtr child;
 	struct FilterStruct s;
@@ -641,7 +654,7 @@ bool Fragment::SavePortion (xmlDocPtr xml, xmlNodePtr node, unsigned start, unsi
 	s.end = end;
 	int charge;
 	if (m_AttrList == NULL)
-		m_AttrList = pango_layout_get_attributes (m_Layout);
+		const_cast <Fragment *> (this)->m_AttrList = pango_layout_get_attributes (m_Layout);
 	pango_attr_list_filter (m_AttrList, (PangoAttrFilterFunc) filter_func, &s);
 	list<PangoAttribute*>::iterator i, iend = s.pal.end ();
 	string str;
@@ -690,7 +703,7 @@ bool Fragment::SavePortion (xmlDocPtr xml, xmlNodePtr node, unsigned start, unsi
 	return true;
 }
 
-xmlNodePtr Fragment::SaveSelection (xmlDocPtr xml)
+xmlNodePtr Fragment::SaveSelection (xmlDocPtr xml) const
 {
 	xmlNodePtr node = xmlNewDocNode (xml, NULL, (xmlChar*) "fragment", NULL);
 	if (!node)
@@ -1018,7 +1031,7 @@ int Fragment::GetElementAtPos (unsigned start, unsigned &end)
 	int Z;
 	char text[4];
 	memset (text, 0, 4);
-	strncpy (text, pango_layout_get_text (m_Layout) + start, 3);
+	strncpy (text, ((m_Layout)?pango_layout_get_text ( m_Layout): m_buf.c_str ()) + start, 3);
 	for (unsigned i = strlen (text); i > 0; i--) {
 		text[i] = 0;
 		if ((Z = Element::Z (text))) {
@@ -1223,4 +1236,135 @@ bool Fragment::GetPosition (double angle, double& x, double& y)
 	return false;
 }
 
+bool Fragment::SetProperty (unsigned property, char const *value)
+{
+	m_bLoading = true;
+	switch (property) {
+	case GCU_PROP_POS2D: {
+		sscanf (value, "%lg %lg", &m_x, &m_y);
+		gcu::Document *doc = GetDocument ();
+		if (doc) {
+			m_x *= doc->GetScale ();
+			m_y *= doc->GetScale ();
+		}
+		m_Atom->SetCoords (m_x, m_y);
+		break;
+	}
+	case GCU_PROP_TEXT_TEXT: {
+		m_buf = value;
+		if (m_EndAtom > m_BeginAtom) {
+			Residue *r = NULL;
+			char sy[Residue::MaxSymbolLength + 1];
+			strncpy (sy, m_buf.c_str () + m_BeginAtom, Residue::MaxSymbolLength);
+			int i = Residue::MaxSymbolLength;
+			while (i > 0) {
+				sy[i] = 0;
+				r = (Residue *) Residue::GetResidue (sy, NULL);
+				if (r)
+					break;
+				i--;
+			}
+			if (r) {
+				m_EndAtom = m_BeginAtom +  + strlen (sy);
+				map<gcu::Atom*, gcu::Bond*>::iterator i;
+				Bond *pBond = (gcp::Bond*) m_Atom->GetFirstBond (i);
+				Atom *pOldAtom = m_Atom;
+				pOldAtom->SetParent (NULL);
+				m_Atom = new FragmentResidue (this, sy);
+				AddChild (m_Atom);
+				m_Atom->SetId ((gchar*) pOldAtom->GetId ());
+				m_Atom->SetCoords (m_x, m_y);
+				if (pBond) {
+					pBond->ReplaceAtom (pOldAtom, m_Atom);
+					m_Atom->AddBond (pBond);
+				}
+				delete pOldAtom;
+			} else {
+				int Z = GetElementAtPos (m_BeginAtom, m_EndAtom);
+				if (Z)
+					m_Atom->SetZ (Z);
+			}
+			Analyze ();
+		}
+		break;
+	}
+	case GCU_PROP_FRAGMENT_ATOM_START:
+		m_BeginAtom = atoi (value);
+		m_EndAtom = m_BeginAtom + Residue::MaxSymbolLength;
+		if (m_buf.length ()) {
+			Residue *r = NULL;
+			char sy[Residue::MaxSymbolLength + 1];
+			strncpy (sy, m_buf.c_str () + m_BeginAtom, Residue::MaxSymbolLength);
+			int i = Residue::MaxSymbolLength;
+			while (i > 0) {
+				sy[i] = 0;
+				r = (Residue *) Residue::GetResidue (sy, NULL);
+				if (r)
+					break;
+				i--;
+			}
+			if (r) {
+				m_EndAtom = m_BeginAtom + strlen (sy);
+				map<gcu::Atom*, gcu::Bond*>::iterator i;
+				Bond *pBond = (gcp::Bond*) m_Atom->GetFirstBond (i);
+				Atom *pOldAtom = m_Atom;
+				pOldAtom->SetParent (NULL);
+				m_Atom = new FragmentResidue (this, sy);
+				AddChild (m_Atom);
+				m_Atom->SetId ((gchar*) pOldAtom->GetId ());
+				m_Atom->SetCoords (m_x, m_y);
+				if (pBond) {
+					pBond->ReplaceAtom (pOldAtom, m_Atom);
+					m_Atom->AddBond (pBond);
+				}
+				delete pOldAtom;
+			} else {
+				int Z = GetElementAtPos (m_BeginAtom, m_EndAtom);
+				if (Z)
+					m_Atom->SetZ (Z);
+			}
+			Analyze ();
+		}
+		break;
+	case GCU_PROP_FRAGMENT_ATOM_ID:
+		m_Atom->SetId (value);
+		if (!m_Atom->GetParent ())
+			AddChild (m_Atom);
+		break;
+	default:
+		break;
+	}
+	m_bLoading = false;
+	return true;
+}
+
+bool Fragment::Analyze () {
+	// search if main atom is at start or at end
+	if ((m_BeginAtom == 0 || m_EndAtom == m_buf.size ()) && ((m_EndAtom - m_BeginAtom) < m_buf.length ()) ){
+		m_Inversable = true;
+	}
+//	int valence = m_Atom->GetValence ();
+	if (m_AttrList != NULL)
+		pango_attr_list_unref (m_AttrList);
+	m_AttrList = pango_attr_list_new ();
+	AnalContent ();
+	return true;
+}
+
+void Fragment::Update () {
+	if (m_Atom->GetBondsNumber () > 0 && m_Inversable) {
+		map<gcu::Atom*, gcu::Bond*>::iterator i;
+		Bond *bond = reinterpret_cast <Bond *> (m_Atom->GetFirstBond (i));
+		double angle = bond->GetAngle2D (m_Atom);
+		if (m_BeginAtom == 0 && (angle < 89. && angle > -89.)) {
+			// FIXME: do something more intelligent since we might have more than two symbols
+			m_buf = m_buf.substr (m_EndAtom);
+			m_BeginAtom = m_buf.length ();
+			m_buf += m_Atom->GetSymbol ();
+			m_EndAtom = m_buf.length ();
+		} else if (angle > 91. || angle < -91.) {
+		}
+	}
+}
+
 }	//	namespace gcp
