@@ -30,6 +30,10 @@
 #include "application.h"
 #include "operation.h"
 #include "theme.h"
+#include <goffice/math/go-math.h>
+#include <cstring>
+
+using namespace gcu;
 
 namespace gcp {
 
@@ -40,14 +44,15 @@ guint ClipboardDataType, ClipboardDataType1;
 bool cleared = true;
 
 GtkTargetEntry const export_targets[] = {
-	{(char *) GCHEMPAINT_ATOM_NAME,  0, 0},
-	{(char *) "image/svg",  0, 1},
-	{(char *) "image/svg+xml",  0, 2},
-	{(char *) "image/png",  0, 3},
-	{(char *) "image/jpeg",  0, 4},
-	{(char *) "image/bmp",  0, 5},
-	{(char *) "UTF8_STRING", 0, 6},
-	{(char *) "STRING", 0, 7}
+	{(char *) GCHEMPAINT_ATOM_NAME,  0, GCP_CLIPBOARD_NATIVE},
+	{(char *) "image/svg",  0, GCP_CLIPBOARD_SVG},
+	{(char *) "image/svg+xml",  0, GCP_CLIPBOARD_SVG_XML},
+	{(char *) "image/x-eps",  0, GCP_CLIPBOARD_EPS},
+	{(char *) "image/png",  0, GCP_CLIPBOARD_PNG},
+	{(char *) "image/jpeg",  0, GCP_CLIPBOARD_JPEG},
+	{(char *) "image/bmp",  0, GCP_CLIPBOARD_BMP},
+	{(char *) "UTF8_STRING", 0, GCP_CLIPBOARD_UTF8_STRING},
+	{(char *) "STRING", 0, GCP_CLIPBOARD_STRING}
 };
 
 void on_receive_targets (GtkClipboard *clipboard, GtkSelectionData *selection_data, Application *App)
@@ -60,6 +65,7 @@ void on_receive_targets (GtkClipboard *clipboard, GtkSelectionData *selection_da
 			GCHEMPAINT_ATOM_NAME,
 			"image/svg",
 			"image/svg+xml",
+			"image/x-eps",
 			"image/png",
 			"image/jpeg",
 			"image/bmp",
@@ -79,7 +85,7 @@ void on_receive_targets (GtkClipboard *clipboard, GtkSelectionData *selection_da
 		}
 
 		gchar* name;
-		*DataType = 7;
+		*DataType = GCP_CLIPBOARD_ALL;
 		for ( j = 0; j < atom_count ; j++) {
 			name = gdk_atom_name (targets [j]);
 			for (i = 0; i < *DataType; i++)
@@ -92,7 +98,7 @@ void on_receive_targets (GtkClipboard *clipboard, GtkSelectionData *selection_da
 	}
 	if (clipboard == sel_clipboard && App != NULL)
 		App->ActivateWindowsActionWidget ("/MainMenu/EditMenu/Paste",
-			ClipboardDataType == 0 || ClipboardDataType == 5 || ClipboardDataType == 6);
+			ClipboardDataType == GCP_CLIPBOARD_NATIVE || ClipboardDataType == GCP_CLIPBOARD_UTF8_STRING || ClipboardDataType == GCP_CLIPBOARD_STRING);
 }
 
 static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info, Application *App)
@@ -108,12 +114,12 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 	*DataType = info;
 	int size;
 	switch (info) {
-	case 0:
+	case GCP_CLIPBOARD_NATIVE:
 		xmlDocDumpFormatMemory (pDoc, &ClipboardData, &size, info);
 		gtk_selection_data_set(selection_data, gdk_atom_intern (GCHEMPAINT_ATOM_NAME, FALSE), 8, (const guchar*) ClipboardData, size);
 		break;
-	case 1:
-	case 2: {
+	case GCP_CLIPBOARD_SVG:
+	case GCP_CLIPBOARD_SVG_XML: {
 		Document *Doc = new Document (NULL, true);
 		View *pView = Doc->GetView ();
 		pView->CreateNewWidget (); // force canvas creation
@@ -125,7 +131,11 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 		delete Doc;
 		break;
 	}
-	case 3: {
+	case GCP_CLIPBOARD_EPS: {
+puts("requesting eps!");
+		break;
+	}
+	case GCP_CLIPBOARD_PNG: {
 		Document *Doc = new Document (NULL, true);
 		View *pView = Doc->GetView ();
 		gsize size;
@@ -138,7 +148,7 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 		delete Doc;
 		break;
 	}
-	case 4: {
+	case GCP_CLIPBOARD_JPEG: {
 		Document *Doc = new Document (NULL, true);
 		View *pView = Doc->GetView ();
 		gsize size;
@@ -151,7 +161,7 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 		delete Doc;
 		break;
 	}
-	case 5: {
+	case GCP_CLIPBOARD_BMP: {
 		Document *Doc = new Document (NULL, true);
 		View *pView = Doc->GetView ();
 		gsize size;
@@ -186,9 +196,9 @@ void on_clear_data (GtkClipboard *clipboard, Application *App)
 	gtk_clipboard_request_contents (clipboard, gdk_atom_intern ("TARGETS", FALSE),  (GtkClipboardReceivedFunc) on_receive_targets, App);
 }
 
-bool WidgetData::IsSelected (Object *obj)
+bool WidgetData::IsSelected (Object const *obj) const
 {
-	std::list<Object*>::iterator i, end = SelectedObjects.end ();
+	std::list<Object*>::const_iterator i, end = SelectedObjects.end ();
 	Object* pGroup = obj->GetGroup ();
 	for (i = SelectedObjects.begin (); i != end; i++)
 		if ((*i == obj) || (*i == pGroup))
@@ -196,7 +206,7 @@ bool WidgetData::IsSelected (Object *obj)
 	return false;
 }
 
-void WidgetData::Unselect (Object* obj)
+void WidgetData::Unselect (Object *obj)
 {
 	SelectedObjects.remove (obj);
 	obj->SetSelected (Canvas, SelStateUnselected);
@@ -214,7 +224,7 @@ void WidgetData::UnselectAll ()
 	}
 }
 
-void WidgetData::SetSelected (Object* obj)
+void WidgetData::SetSelected (Object *obj)
 {
 	if (!IsSelected (obj)) {
 		SelectedObjects.push_front (obj);
@@ -247,6 +257,8 @@ void WidgetData::MoveItems (Object* obj, double dx, double dy)
 
 void WidgetData::MoveSelection (double dx, double dy)
 {
+	if (!SelectedObjects.size ())
+		return;
 	std::list<Object*>::iterator i, end = SelectedObjects.end ();
 	Document* pDoc = m_View->GetDoc ();
 	Operation* pOp = pDoc-> GetNewOperation (GCP_MODIFY_OPERATION);
@@ -296,14 +308,16 @@ void WidgetData::Copy (GtkClipboard* clipboard)
 	gtk_clipboard_request_contents (clipboard, gdk_atom_intern ("TARGETS", FALSE),  (GtkClipboardReceivedFunc) on_receive_targets, App);
 }
 
-void WidgetData::GetObjectBounds (Object* obj, ArtDRect &rect)
+void WidgetData::GetObjectBounds (Object const *obj, ArtDRect &rect) const
 {
-	Object* pObject;
-	GnomeCanvasGroup* group;
+	Object const *pObject;
+	GnomeCanvasGroup const *group;
+	std::map<gcu::Object const *, GnomeCanvasGroup*>::const_iterator g;
 	double x1, y1, x2,  y2;
-	if ((group = Items[obj])) {
+	if ((g = Items.find (obj)) != Items.end ()) {
+		group = (*g).second;
 		gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (group), &x1, &y1, &x2, &y2);
-		if (rect.x0 < -9.) {
+		if (!go_finite (rect.x0)) {
 			rect.x0 = x1;
 			rect.y0 = y1;
 			rect.x1 = x2;
@@ -314,9 +328,8 @@ void WidgetData::GetObjectBounds (Object* obj, ArtDRect &rect)
 			if (rect.x1 < x2) rect.x1 = x2;
 			if (rect.y1 < y2) rect.y1 = y2;
 		}
-	} else
-		Items.erase (obj);
-	std::map<std::string, Object*>::iterator i;
+	}
+	std::map<std::string, Object*>::const_iterator i;
 	pObject = obj->GetFirstChild (i);
 	while (pObject) {
 		GetObjectBounds (pObject, rect);
@@ -324,23 +337,27 @@ void WidgetData::GetObjectBounds (Object* obj, ArtDRect &rect)
 	}
 }
 
-void WidgetData::GetSelectionBounds (ArtDRect &rect)
+void WidgetData::GetSelectionBounds (ArtDRect &rect) const
 {
-	std::list<Object*>::iterator i, end = SelectedObjects.end ();
-	rect.x0 = -10.;
+	std::list<Object*>::const_iterator i, end = SelectedObjects.end ();
+	rect.x0 = go_nan;
 	for (i = SelectedObjects.begin (); i != end; i++)
 		GetObjectBounds (*i, rect);
+	if (!go_finite (rect.x0))
+		rect.x0 = rect.y0 = rect.x1 = rect.y1 = 0.;
 }
 
-void WidgetData::GetObjectBounds (Object* obj, ArtDRect *rect)
+void WidgetData::GetObjectBounds (Object const *obj, ArtDRect *rect) const
 {
-	rect->x0 = -10.;
+	rect->x0 = go_nan;
 	GetObjectBounds (obj, *rect);
+	if (!go_finite (rect->x0))
+		rect->x0 = rect->y0 = rect->x1 = rect->y1 = 0.;
 }
 
 void WidgetData::SelectAll ()
 {
-	std::map<Object*, GnomeCanvasGroup*>::iterator i, end = Items.end ();
+	std::map<Object const*, GnomeCanvasGroup*>::iterator i, end = Items.end ();
 	Object *pObject;
 	for (i = Items.begin (); i != end; i++) {
 		pObject = (*i).first->GetGroup ();
@@ -349,7 +366,7 @@ void WidgetData::SelectAll ()
 				SetSelected (pObject);
 		}
 		else if (!IsSelected ((*i).first))
-			SetSelected ((*i).first);
+			SetSelected (const_cast <Object *> ((*i).first));
 	}
 }
 
