@@ -28,8 +28,8 @@
 #include "loader.h"
 #include <goffice/goffice.h>
 #include <goffice/app/io-context.h>
-#include <gsf-gnome/gsf-input-gnomevfs.h>
-#include <gsf-gnome/gsf-output-gnomevfs.h>
+#include <gsf/gsf-input-gio.h>
+#include <gsf/gsf-output-gio.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
@@ -59,7 +59,7 @@ Application::Application (string name, string datadir, char const *help_name, ch
 {
 #ifdef HAVE_GO_CONF_SYNC
 	if (m_ConfDir == NULL) {
-		libgoffice_init ("gchemutils");
+		libgoffice_init ();
 		m_ConfDir = go_conf_get_node (NULL, GCU_CONF_DIR);
 	}
 #else
@@ -141,6 +141,7 @@ Application::~Application ()
 	g_object_unref (m_RecentManager);
 	Apps.erase (this);
 	if (Apps.empty ()) {
+		ClearDialogs (); // needed to cleanly stop goffice
 #ifdef HAVE_GO_CONF_SYNC
 		go_conf_free_node (m_ConfDir);
 		m_ConfDir = NULL;
@@ -285,11 +286,12 @@ bool Application::Load (std::string const &uri, const gchar *mime_type, Document
 	if (!l)
 		return false;
 	string old_num_locale = setlocale (LC_NUMERIC, NULL);
-	setlocale(LC_NUMERIC, "C");
+	setlocale (LC_NUMERIC, "C");
 	GError *error = NULL;
-	GsfInput *input = gsf_input_gnomevfs_new (uri.c_str (), &error);
+	GsfInput *input = gsf_input_gio_new_for_uri (uri.c_str (), &error);
 	if (error) {
 		g_error_free (error);
+		return false;
 	}
 	IOContext *io = gnumeric_io_context_new (gcu_get_cmd_context ());
 	bool ret = l->Read (Doc, input, mime_type, io);
@@ -305,9 +307,9 @@ bool Application::Save (std::string const &uri, const gchar *mime_type, Document
 	if (!l)
 		return false;
 	string old_num_locale = setlocale (LC_NUMERIC, NULL);
-	setlocale(LC_NUMERIC, "C");
+	setlocale (LC_NUMERIC, "C");
 	GError *error = NULL;
-	GsfOutput *output = gsf_output_gnomevfs_new (uri.c_str (), &error);
+	GsfOutput *output = gsf_output_gio_new_for_uri (uri.c_str (), &error);
 	if (error) {
 		g_error_free (error);
 	}
@@ -323,7 +325,7 @@ bool Application::Save (std::string const &uri, const gchar *mime_type, Document
 GOConfNode *Application::GetConfDir ()
 {
 	if (m_ConfDir == NULL) {
-		libgoffice_init ("gchemutils");
+		libgoffice_init ();
 		m_ConfDir = go_conf_get_node (NULL, GCU_CONF_DIR);
 	}
 	return m_ConfDir;
