@@ -25,15 +25,17 @@
 #ifndef GCU_ATOM_H
 #define GCU_ATOM_H
 
-#include <map>
-#include <glib.h>
 #include "object.h"
+#include <glib.h>
+#include <map>
+#include <vector>
 
 namespace gcu
 {
 	
 class Bond;
 class Cycle;
+class AtomMatchState;
 
 /*!\class Atom gcu/atom.h
 This class is used to represent atoms.
@@ -92,7 +94,7 @@ Multiplies all three coordinates of this Atom by ZoomFactor.
 Retrieves the coordinates of this Atom.
 @return true if successful and false if an error occurs (if x or y is NULL).
 */
-	virtual bool GetCoords (double *x, double *y, double *z = NULL);
+	virtual bool GetCoords (double *x, double *y, double *z = NULL) const;
 /*!
 @param x: the new x coordinate of the Atom.
 @param y: the new y coordinate of the Atom.
@@ -104,7 +106,7 @@ Changes the position of this Atom.
 /*!
 @return the atomic number of the atom.
 */
-	int GetZ () {return m_Z;}
+	int GetZ () const {return m_Z;}
 /*!
 @param Z: the new atomic number of the Atom.
 
@@ -125,7 +127,7 @@ Changes the formal charge of this Atom.
 /*!
 @return the atomic symbol of this Atom or NULL if the element is unknown.
 */
-	virtual const gchar* GetSymbol ();
+	virtual const gchar* GetSymbol () const;
 /*!
 @param pBond: a pointer to the new Bond to this Atom.
 
@@ -141,15 +143,15 @@ Removes a Bond.
 /*!
 @return the x coordinate of this Atom.
 */
-	double x () {return m_x;}
+	double x () const {return m_x;}
 /*!
 @return the y coordinate of this Atom.
 */
-	double y () {return m_y;}
+	double y () const {return m_y;}
 /*!
 @return the z coordinate of this Atom.
 */
-	double z () {return m_z;}
+	double z () const {return m_z;}
 /*!
 @param i: a C++ std::map iterator.
 
@@ -168,16 +170,16 @@ Use this method to iterate through the list of Bond instances of this Atom.
 @param pAtom: a pointer to an Atom instance.
 @return a pointer to the Bond shared by pAtom and this Atom if it exists or NULL.
 */
-	Bond* GetBond (Atom* pAtom);
+	Bond* GetBond (Atom* pAtom) const;
 /*!
 @return the number of Bond instances shared by this Atom. It does not take multiplicity of bonds into account.
 */
-	int GetBondsNumber () {return m_Bonds.size();}
+	int GetBondsNumber () const {return m_Bonds.size();}
 /*!
 @param xml: the xmlDoc used to save the document.
 @return a pointer to the xmlNode representing this Atom or NULL if an error occured.
 */
-	virtual xmlNodePtr Save (xmlDocPtr xml);
+	virtual xmlNodePtr Save (xmlDocPtr xml) const;
 /*!
 @param node: a pointer to the xmlNode containing the serialized Atom.
 
@@ -198,7 +200,7 @@ It might be overrided for derived class when it is not convenient to override th
 This virtual method is called at the end of the Atom::Save method. The default behavior is to do nothing.
 It might be overrided for derived class when it is not convenient to override the Atom::Save method.
 */
-	virtual bool SaveNode (xmlDocPtr xml, xmlNodePtr node);
+	virtual bool SaveNode (xmlDocPtr xml, xmlNodePtr node) const;
 /*!
 @param x: the x component of the transation vector.
 @param y: the y component of the transation vector.
@@ -216,9 +218,22 @@ Used to move and/or transform an atom.
 */
 	virtual void Transform2D (Matrix2D& m, double x, double y);
 
+/*!
+*/
 	bool SetProperty (unsigned property, char const *value);
+
+/*!
+*/
+	std::string GetProperty (unsigned property) const;
+
+/*!
+*/
 	bool IsInCycle (Cycle* pCycle);
 
+/*!
+*/
+	virtual bool Match (Atom *atom, AtomMatchState &state);
+
 protected:
 /*!
 The atomic number of the Atom.
@@ -246,5 +261,18 @@ The Bond instances of the Atom. The index of the map is a pointer to the other e
 	std::map<Atom*, Bond*> m_Bonds;
 };
 
+class AtomPair {
+	public:
+	AtomPair (Atom *at1, Atom *at2) {atom1 = at1; atom2 = at2;}
+	Atom *atom1, *atom2;
+};
+
+class AtomMatchState
+{
+public:
+	std::map <Atom*, int>  mol1, mol2;
+	std::vector <AtomPair> atoms;
+};
+
 } //namespace gcu
 #endif // GCU_ATOM_H
