@@ -79,10 +79,6 @@ private:
 	map <string, Element*> EltsMap;
 };
 
-} // namespace gcu
-
-using namespace gcu;
-
 EltTable Table;
 
 EltTable::EltTable()
@@ -103,21 +99,24 @@ EltTable::EltTable()
 	Langs["it"] = _("Italian");
 	Langs["pl"] = _("Polish");
 	Langs["ru"] = _("Russian");
-	if (!(xml = xmlParseFile (PKGDATADIR"/elements.xml")))
-	{
+	if (!(xml = xmlParseFile (PKGDATADIR"/elements.xml"))) {
 		g_error (_("Can't find and read elements.xml"));
 	}
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	xmlNode* node = xml->children, *child;
-	if (strcmp ((const char*) node->name, "gpdata")) g_error (_("Incorrect file format: elements.xml"));
+	if (strcmp ((const char*) node->name, "gpdata")) {
+		g_error (_("Incorrect file format: elements.xml"));
+	}
 	node = node->children;
 	Element* Elt;
 	while (node)
 	{
 		if (strcmp ((const char*) node->name, "text"))
 		{
-			if (strcmp ((const char*) node->name, "element")) g_error (_("Incorrect file format: elements.xml"));
+			if (strcmp ((const char*) node->name, "element")) {
+				g_error (_("Incorrect file format: elements.xml"));
+			}
 			buf = (char*) xmlGetProp (node, (xmlChar*) "symbol");
 			num = (char*) xmlGetProp (node, (xmlChar*) "Z");
 			Elt = new Element (Z = atoi (num), buf);
@@ -126,6 +125,12 @@ EltTable::EltTable()
 			num = (char*) xmlGetProp (node, (xmlChar*) "max_bonds");
 			Elt->m_MaxBonds = atoi (num);
 			xmlFree (num);
+			num = (char*) xmlGetProp (node, (xmlChar*) "valence");
+			if (num) {
+				Elt->m_DefaultValence = atoi (num);
+				xmlFree (num);
+			} else
+				Elt->m_DefaultValence = -1;
 			child = node->children;
 			DefaultName = NULL;
 			while (child)
@@ -211,7 +216,8 @@ void EltTable::AddElement(Element* Elt)
 }
 
 Element::Element(int Z, const char* Symbol):
-	m_AtomicWeight (NULL)
+	m_AtomicWeight (NULL),
+	m_MetallicCached (false)
 {
 	m_Z = Z;
 	strncpy(m_Symbol, Symbol, 3);
@@ -220,43 +226,15 @@ Element::Element(int Z, const char* Symbol):
 	m_BestSide = true;
 	switch (m_Z)
 	{
-		case 6:
-		case 14:
-		case 32:
-			m_DefaultValence = 4;
-			break;
-		case 5:
-		case 7:
-		case 13:
-		case 15:
-		case 33:
-		case 51:
-			m_DefaultValence = 3;
-			break;
 		case 8:
 		case 16:
 		case 34:
 		case 52:
-			m_BestSide = false;
-		case 4:
-			m_DefaultValence = 2;
-			break;
 		case 9:
 		case 17:
 		case 35:
 		case 53:
 			m_BestSide = false;
-			m_DefaultValence = 1;
-			break;
-		case 2:
-		case 10:
-		case 28:
-		case 36:
-		case 54:
-			m_DefaultValence = 0;
-			break;
-		default:
-			m_DefaultValence = -1;
 	}
 	m_DefaultColor[0] = m_DefaultColor[1] = m_DefaultColor[2] = 0.0;
 	if (m_Z <= 2) {
@@ -361,24 +339,24 @@ unsigned Element::GetMaxBonds(gint Z)
 	return (Elt)? Elt->GetMaxBonds(): 0;
 }
 
-bool Element::GetRadius(GcuAtomicRadius* radius)
+bool Element::GetRadius (GcuAtomicRadius* radius)
 {
 	Element* Elt = Table[radius->Z];
 	if (!Elt || !Elt->m_radii.size ())
 		return false;
-	for (int i = 0; Elt->m_radii[i]; i++)
-	{
-		if (radius->type != Elt->m_radii[i]->type) continue;
-		if (radius->charge != Elt->m_radii[i]->charge) continue;
-		if ((radius->cn >= 0) &&(radius->cn != Elt->m_radii[i]->cn)) continue;
-		if ((radius->spin != GCU_N_A_SPIN) &&(radius->spin != Elt->m_radii[i]->spin)) continue;
-		if (!radius->scale)
-		{
+	for (int i = 0; Elt->m_radii[i]; i++) {
+		if (radius->type != Elt->m_radii[i]->type)
+			continue;
+		if (radius->charge != Elt->m_radii[i]->charge)
+			continue;
+		if ((radius->cn >= 0) &&(radius->cn != Elt->m_radii[i]->cn))
+			continue;
+		if ((radius->spin != GCU_N_A_SPIN) &&(radius->spin != Elt->m_radii[i]->spin))
+			continue;
+		if (!radius->scale) {
 			*radius = *Elt->m_radii[i];
 			return true;
-		}
-		else if (!strcmp(radius->scale, Elt->m_radii[i]->scale))
-		{
+		} else if (Elt->m_radii[i]->scale && !strcmp(radius->scale, Elt->m_radii[i]->scale)) {
 			radius->value = Elt->m_radii[i]->value;
 			return true;
 		}
@@ -423,14 +401,16 @@ void Element::LoadRadii ()
 	static bool loaded = false;
 	if (loaded)
 		return;
-	if (!(xml = xmlParseFile (PKGDATADIR"/radii.xml")))
-	{
+	LoadBODR ();
+	if (!(xml = xmlParseFile (PKGDATADIR"/radii.xml"))) {
 		g_error (_("Can't find and read radii.xml"));
 	}
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	xmlNode* node = xml->children, *child;
-	if (strcmp ((const char*) node->name, "gpdata")) g_error (_("Incorrect file format: radii.xml"));
+	if (strcmp ((const char*) node->name, "gpdata")) {
+		g_error (_("Incorrect file format: radii.xml"));
+	}
 	node = node->children;
 	Element* Elt;
 	set<string>::iterator it = units.find ("pm");
@@ -441,9 +421,15 @@ void Element::LoadRadii ()
 	while (node) {
 		if (strcmp ((const char*) node->name, "text"))
 		{
-			if (strcmp ((const char*) node->name, "element")) g_error (_("Incorrect file format: radii.xml"));
+			if (strcmp ((const char*) node->name, "element")) {
+				g_error (_("Incorrect file format: radii.xml"));
+			}
 			num = (char*) xmlGetProp (node, (xmlChar*) "Z");
 			Elt = Table[Z = atoi (num)];
+			if (!Elt) {
+				node = node->next;
+				continue;
+			}
 			child = node->children;
 			while (child)
 			{
@@ -456,15 +442,14 @@ void Element::LoadRadii ()
 					radius->Z = Z;	//FIXME: is it really useful there?
 					buf = (char*) xmlGetProp (child, (xmlChar*) "type");
 					if (!buf ||
-						((!((!strcmp (buf, "covalent")) && (radius->type = GCU_COVALENT))) &&
-						(!((!strcmp (buf, "vdW")) && (radius->type = GCU_VAN_DER_WAALS))) &&
-						(!((!strcmp (buf, "ionic")) && (radius->type = GCU_IONIC))) &&
+						((!((!strcmp (buf, "ionic")) && (radius->type = GCU_IONIC))) &&
 						(!((!strcmp (buf, "metallic")) && (radius->type = GCU_METALLIC))) &&
 						(!((!strcmp (buf, "atomic")) && ((radius->type = GCU_ATOMIC) || true))))) {
 						//invalid radius
 						delete radius;
 						if (buf)
 							xmlFree (buf);
+						child = child->next;
 						continue;
 					}
 					buf = (char*) xmlGetProp (child, (xmlChar*) "scale");
@@ -500,8 +485,9 @@ void Element::LoadRadii ()
 						xmlFree (buf);
 					} else
 						delete radius;
-				} else
+				} else {
 					g_error ("Invalid radius node");
+				}
 				child = child->next;
 			}
 			Elt->m_radii.push_back (NULL);
@@ -523,19 +509,22 @@ void Element::LoadElectronicProps ()
 	static bool loaded = false;
 	if (loaded)
 		return;
-	if (!(xml = xmlParseFile (PKGDATADIR"/elecprops.xml")))
-	{
+	if (!(xml = xmlParseFile (PKGDATADIR"/elecprops.xml"))) {
 		g_error (_("Can't find and read elecprops.xml"));
 	}
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	xmlNode* node = xml->children, *child;
-	if (strcmp ((const char*) node->name, "gpdata")) g_error (_("Incorrect file format: elecprops.xml"));
+	if (strcmp ((const char*) node->name, "gpdata")) {
+		g_error (_("Incorrect file format: elecprops.xml"));
+	}
 	node = node->children;
 	Element* Elt;
 	while (node) {
 		if (strcmp ((const char*) node->name, "text")) {
-			if (strcmp ((const char*) node->name, "element")) g_error (_("Incorrect file format: elecprops.xml"));
+			if (strcmp ((const char*) node->name, "element")) {
+				g_error (_("Incorrect file format: elecprops.xml"));
+			}
 			num = (char*) xmlGetProp (node, (xmlChar*) "Z");
 			Elt = Table[Z = atoi (num)];
 			child = node->children;
@@ -659,8 +648,9 @@ void Element::LoadElectronicProps ()
 						xmlFree (buf);
 					} else
 						Elt->m_ae[rank].unit = "kJ.mol<sup>-1</sup>";
-				} else
+				} else {
 					g_error ("Invalid property node");
+				}
 				child = child->next;
 			}
 			Elt->m_en.push_back (NULL);
@@ -681,21 +671,24 @@ void Element::LoadIsotopes ()
 	static bool loaded = false;
 	if (loaded)
 		return;
-	if (!(xml = xmlParseFile (PKGDATADIR"/isotopes.xml")))
-	{
+	if (!(xml = xmlParseFile (PKGDATADIR"/isotopes.xml"))) {
 		g_error (_("Can't find and read isotopes.xml"));
 	}
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	xmlNode* node = xml->children, *child;
-	if (strcmp ((const char*) node->name, "gpdata")) g_error (_("Incorrect file format: isotopes.xml"));
+	if (strcmp ((const char*) node->name, "gpdata")) {
+		g_error (_("Incorrect file format: isotopes.xml"));
+	}
 	node = node->children;
 	Element *Elt;
 	Isotope *Is;
 	int minA, maxA, niso;
 	while (node) {
 		if (strcmp ((const char*) node->name, "text")) {
-			if (strcmp ((const char*) node->name, "element")) g_error (_("Incorrect file format: isotopes.xml"));
+			if (strcmp ((const char*) node->name, "element")) {
+				g_error (_("Incorrect file format: isotopes.xml"));
+			}
 			minA = maxA = niso = 0;
 			num = (char*) xmlGetProp (node, (xmlChar*) "Z");
 			Elt = Table[Z = atoi (num)];
@@ -822,12 +815,21 @@ GcuDimensionalValue const *Element::GetElectronAffinity (unsigned rank)
 
 void Element::LoadBODR ()
 {
+	static bool loaded = false;
+	if (loaded)
+		return;
+	loaded = true;
 	char *old_num_locale;
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	xmlDocPtr xml = xmlParseFile (BODR_PKGDATADIR"/elements.xml");
 	if (!xml)
 		return;
+	set<string>::iterator it = units.find ("pm");
+	if (it == units.end ()) {
+		units.insert ("pm");
+		it = units.find ("pm");
+	}
 	xmlNodePtr node = xml->children, child;
 	char *buf = NULL, *unit;
 	Element *elt;
@@ -929,6 +931,57 @@ void Element::LoadBODR ()
 					child = child->next;
 				}
 				if (elt) {
+					DimensionalValue const *v = dynamic_cast <DimensionalValue const *> (elt->GetProperty ("radiusVDW"));
+					double r;
+					if (v) {
+						r = v->GetAsDouble ();
+						GcuDimensionalValue const val = v->GetValue ();
+						int prec = val.prec;
+						if (!strcmp (val.unit, "Å")) {
+							r *= 100.;
+							prec = (prec > 2)? prec - 2: 0;
+						} else if (!strcmp (val.unit, "pm"))
+							r = -1.; // FIXME: allow other units if needed
+						if (r > 0.) {
+							GcuAtomicRadius* radius = new GcuAtomicRadius;
+							radius->value.value = r;
+							radius->value.prec = prec;
+							radius->value.delta = val.delta;
+							radius->value.unit = "pm";
+							radius->Z = elt->GetZ ();
+							radius->type = GCU_VAN_DER_WAALS;
+							radius->charge = 0;
+							radius->scale = NULL;
+							radius->cn = -1;
+							radius->spin = GCU_N_A_SPIN;
+							elt->m_radii.push_back (radius);
+						}
+					}
+					v = dynamic_cast <DimensionalValue const *> (elt->GetProperty ("radiusCovalent"));
+					if (v) {
+						r = v->GetAsDouble ();
+						GcuDimensionalValue const val = v->GetValue ();
+						int prec = val.prec;
+						if (!strcmp (val.unit, "Å")) {
+							r *= 100.;
+							prec = (prec > 2)? prec - 2: 0;
+						} else if (!strcmp (val.unit, "pm"))
+							r = -1.; // FIXME: allow other units if needed
+						if (r > 0.) {
+							GcuAtomicRadius* radius = new GcuAtomicRadius;
+							radius->value.value = r;
+							radius->value.prec = prec;
+							radius->value.delta = val.delta;
+							radius->value.unit = "pm";
+							radius->Z = elt->GetZ ();
+							radius->type = GCU_COVALENT;
+							radius->charge = 0;
+							radius->scale = NULL;
+							radius->cn = -1;
+							radius->spin = GCU_N_A_SPIN;
+							elt->m_radii.push_back (radius);
+						}
+					}
 				}
 			}
 			node = node->next;
@@ -939,15 +992,47 @@ void Element::LoadBODR ()
 	g_free (old_num_locale);
 }
 
+Value const *Element::GetProperty (char const *property_name)
+{
+	map<string, Value*>::iterator i = props.find (property_name);
+	return (i == props.end ())? NULL: (*i).second;
+}
+
 int Element::GetIntegerProperty (char const *property_name)
 {
 	map<string, int>::iterator i = iprops.find (property_name);
 	return (i == iprops.end ())? GCU_ERROR: (*i).second;
 }
 
+string &Element::GetStringProperty (char const *property_name)
+{
+	static string empty_string;
+	map<string, string>::iterator i = sprops.find (property_name);
+	return (i == sprops.end ())? empty_string: (*i).second;
+}
+
 DimensionalValue const *Element::GetWeight ()
 {
 	if (m_AtomicWeight == NULL)
 		m_AtomicWeight = dynamic_cast<DimensionalValue const*> (props["mass"]);
 	return m_AtomicWeight;
 }
+
+bool Element::IsMetallic ()
+{
+	if (!m_MetallicCached) {
+		LoadRadii ();
+		GcuAtomicRadius r;
+		r.Z = m_Z;
+		r.type = GCU_METALLIC;
+		r.charge = 0;
+		r.spin = GCU_N_A_SPIN;
+		r.cn = -1;
+		r.scale = NULL;
+		m_Metallic = GetRadius (&r) || (m_Z > 99 && m_Z < 118);
+		m_MetallicCached = true;
+	}
+	return m_Metallic;
+}
+
+} // namespace gcu
