@@ -26,7 +26,9 @@
 #include "document.h"
 #include "macros.h"
 #include "printable.h"
-#include <complex>
+#include <goffice/graph/gog-series.h>
+#include <string>
+#include <vector>
 
 namespace gcu
 {
@@ -39,6 +41,9 @@ typedef enum {
 	GCU_SPECTRUM_INFRARED_TRANSFORMED,
 	GCU_SPECTRUM_UV_VISIBLE,
 	GCU_SPECTRUM_NMR,
+	GCU_SPECTRUM_NMR_FID,
+	GCU_SPECTRUM_NMR_PEAK_TABLE,
+	GCU_SPECTRUM_NMR_PEAK_ASSIGNMENTS,
 	GCU_SPECTRUM_MASS,
 	GCU_SPECTRUM_MAX
 } SpectrumType;
@@ -48,14 +53,46 @@ typedef enum {
 	GCU_SPECTRUM_UNIT_TRANSMITTANCE,
 	GCU_SPECTRUM_UNIT_ABSORBANCE,
 	GCU_SPECTRUM_UNIT_PPM,
-	GCU_SPECTRUM_NANOMETERS,
-	GCU_SPECTRUM_MICROMETERS,
+	GCU_SPECTRUM_UNIT_NANOMETERS,
+	GCU_SPECTRUM_UNIT_MICROMETERS,
+	GCU_SPECTRUM_UNIT_SECONDS,
+	GCU_SPECTRUM_UNIT_HZ,
+	GCU_SPECTRUM_UNIT_M_Z,
+	GCU_SPECTRUM_UNIT_REL_ABUNDANCE,
 	GCU_SPECTRUM_UNIT_MAX
 } SpectrumUnitType;
 
+typedef enum {
+	GCU_SPECTRUM_TYPE_INDEPENDENT,
+	GCU_SPECTRUM_TYPE_DEPENDENT,
+	GCU_SPECTRUM_TYPE_PAGE,
+	GCU_SPECTRUM_TYPE_MAX
+} SpectrumVarType;
+
+typedef enum {
+	GCU_SPECTRUM_FORMAT_ASDF,
+	GCU_SPECTRUM_FORMAT_AFFN,
+	GCU_SPECTRUM_FORMAT_PAC,
+	GCU_SPECTRUM_FORMAT_SQZ,
+	GCU_SPECTRUM_FORMAT_DIF,
+	GCU_SPECTRUM_FORMAT_MAX
+} SpectrumFormatType;
+
 class Application;
 class SpectrumView;
 
+typedef struct  {
+	std::string Name;
+	char Symbol;
+	SpectrumVarType Type;
+	SpectrumUnitType Unit;
+	SpectrumFormatType Format;
+	unsigned NbValues;
+	double First, Last, Min, Max, Factor;
+	double *Values;
+	GogSeries *Series;
+} JdxVar;
+
 class SpectrumDocument: public Document, public Printable
 {
 public:
@@ -83,25 +120,42 @@ Loads a spaectrum from the provided uri. Default mime type is NULL,
 */
 	void Load (char const *uri, char const *mime_type = NULL);
 
+/*!
+*/
+	void OnUnitChanged (int i);
+
+/*!
+*/
+	void OnShowIntegral ();
+
+/*!
+*/
+	void OnTransformFID (GtkButton *btn);
+
 private:
 	void LoadJcampDx (char const *data);
 	void ReadDataLine (char const *data, std::list<double> &l);
-	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context);
+	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const;
 	GtkWindow *GetGtkWindow ();
+	void ReadDataTable (std::istream &s, double *x, double *y);
+	double GetConversionFactor (SpectrumUnitType oldu, SpectrumUnitType newu);
 
 private:
 	double *x, *y;
-	std::complex<double> *z;
 	unsigned npoints;
 	double maxx, maxy, minx, miny;
 	double firstx, lastx, deltax, firsty;
 	double xfactor, yfactor;
+	std::vector <JdxVar> variables;
+	int X, Y, R, I, integral, Rt, It, Rp;
+	double freq;
 
 GCU_PROT_PROP (SpectrumView*, View)
 GCU_RO_PROP (bool, Empty)
 GCU_RO_PROP (SpectrumType, SpectrumType)
 GCU_RO_PROP (SpectrumUnitType, XUnit)
 GCU_RO_PROP (SpectrumUnitType, YUnit)
+GCU_RO_PROP (bool, IntegralVisible)
 };
 
 }
