@@ -24,6 +24,8 @@
 
 #include "config.h"
 #include "gchemtable-app.h"
+#include "gchemtable-curve.h"
+#include "gchemtable-data.h"
 #include "gchemtable-elt.h"
 #include <gcu/chemistry.h>
 #include <gcu/element.h>
@@ -38,12 +40,18 @@
 #include <cstdlib>
 #include <cstring>
 
-static void on_quit (GtkWidget *widget, void *data)
+static void on_quit (GtkWidget *widget, GChemTableApp *App)
 {
+	delete App;
 	gtk_main_quit();
 }
 
-void on_no_colors (GtkWidget *widget, GChemTableApp *App)
+static void on_new_chart (GtkWidget *widget, GChemTableApp *App)
+{
+	App->OnNewChart ();
+}
+
+static void on_no_colors (GtkWidget *widget, GChemTableApp *App)
 {
 	App->SetColorScheme ("none");
 }
@@ -53,16 +61,36 @@ void on_default_colors (GtkWidget *widget, GChemTableApp *App)
 	App->SetColorScheme ("default");
 }
 
-void on_state_colors (GtkWidget *widget, GChemTableApp *App)
+static void on_state_colors (GtkWidget *widget, GChemTableApp *App)
 {
 	App->SetColorScheme ("state");
 }
 
-void on_family_colors (GtkWidget *widget, GChemTableApp *App)
+static void on_family_colors (GtkWidget *widget, GChemTableApp *App)
 {
 	App->SetColorScheme ("family");
 }
 
+void on_acidity_colors (GtkWidget *widget, GChemTableApp *App)
+{
+	App->SetColorScheme ("acidity");
+}
+
+void on_electroneg_colors (GtkWidget *widget, GChemTableApp *App)
+{
+	App->SetColorScheme ("electroneg");
+}
+
+static void on_radius_colors (GtkWidget *widget, GChemTableApp *App)
+{
+	App->SetColorScheme ("radius");
+}
+
+static void on_block_colors (GtkWidget *widget, GChemTableApp *App)
+{
+	App->SetColorScheme ("block");
+}
+
 static void on_about_activate_url (GtkAboutDialog *about, const gchar *url, gpointer data)
 {
 	GnomeVFSResult error = gnome_vfs_url_show(url);
@@ -93,47 +121,18 @@ static void on_bug (GtkWidget *widget, GChemTableApp *app)
 
 static void on_about (GtkWidget *widget, GChemTableApp *app)
 {
-	const gchar * authors[] = {"Jean Bréfort", NULL};
-	const gchar * comments = _("GChemTable is a chemical periodic table of the elements application");
-	/* const gchar * documentors[] = {NULL}; */
-	const gchar * copyright = _("Copyright © 2005-2007 Jean Bréfort");
-	const gchar * license =
-		"This program is free software; you can redistribute it and/or\n"
-		"modify it under the terms of the GNU General Public License as\n"
-		"published by the Free Software Foundation; either version 2 of the\n"
-		"License, or (at your option) any later version.\n\n"
-		"This program is distributed in the hope that it will be useful,\n"
-		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-		"GNU General Public License for more details.\n\n"
-		"You should have received a copy of the GNU General Public License\n"
-		"along with this program; if not, write to the Free Software\n"
-		"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307\n"
-		"USA";
-	
-	gtk_about_dialog_set_url_hook(on_about_activate_url, NULL, NULL);
-
-	/* Note to translators: replace the following string with the appropriate credits for you lang */
-	const gchar * translator_credits = _("translator_credits");
-	gtk_show_about_dialog (app->GetWindow (),
-	                       "name", "GChemTable",
-	                       "authors", authors,
-	                       "comments", comments,
-	                       "copyright", copyright,
-	                       "license", license,
-	                       "translator_credits", translator_credits,
-	                       "version", VERSION,
-	                       "website", "http://www.nongnu.org/gchemutils",
-	                       NULL);
+	app->OnAbout ();
 }
 
-void on_changed (GtkPeriodic* periodic, guint Z, GChemTableApp *app)
+static void on_changed (GtkPeriodic* periodic, guint Z, GChemTableApp *app)
 {
 	app->OnElement (Z);
 }
 
 static GtkActionEntry entries[] = {
   { "FileMenu", NULL, N_("_File") },
+	  { "NewChart", NULL, N_("New _Chart"), "<control>N",
+		  N_("Create a new chart"), G_CALLBACK (on_new_chart) },
 	  { "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
 		  N_("Quit GChemTable"), G_CALLBACK (on_quit) },
   { "ViewMenu", NULL, N_("_View") },
@@ -148,6 +147,18 @@ static GtkActionEntry entries[] = {
 	  {"FamilyColors", NULL, N_("Family"), NULL,
 		  N_("Use colors to display the family grouping of the elements"),
                   G_CALLBACK (on_family_colors) },
+	  {"AcidityColors", NULL, N_("Acidity"), NULL,
+		  N_("Use colors to display the acidity of the elements"),
+                  G_CALLBACK (on_acidity_colors) },
+	  {"ElectronegColors", NULL, N_("Electronegativity"), NULL,
+		  N_("Use colors to display the electronegativity of the elements"),
+                  G_CALLBACK (on_electroneg_colors) },
+	  {"RadiusColors", NULL, N_("Atomic radius"), NULL,
+		  N_("Use colors to display the covalent radii of the elements"),
+                  G_CALLBACK (on_radius_colors) },
+	  {"BlockColors", NULL, N_("Block"), NULL,
+		  N_("Use colors to display the blocks elements belong to"),
+                  G_CALLBACK (on_block_colors) },
   { "HelpMenu", NULL, N_("_Help") },
 	  { "Help", GTK_STOCK_HELP, N_("_Contents"), "F1",
 		  N_("View help for the Periodic Table"), G_CALLBACK (on_help) },
@@ -165,6 +176,9 @@ static const char *ui_description =
 "<ui>"
 "  <menubar name='MainMenu'>"
 "    <menu action='FileMenu'>"
+"	   <separator name='file-sep1'/>"
+"      <menuitem action='NewChart'/>"
+"	   <separator name='file-sep2'/>"
 "      <menuitem action='Quit'/>"
 "    </menu>"
 "    <menu action='ViewMenu'>"
@@ -173,6 +187,10 @@ static const char *ui_description =
 "        <menuitem action='DefaultColors'/>"
 "        <menuitem action='StateColors'/>"
 "        <menuitem action='FamilyColors'/>"
+//"        <menuitem action='AcidityColors'/>"
+"        <menuitem action='ElectronegColors'/>"
+"        <menuitem action='RadiusColors'/>"
+"        <menuitem action='BlockColors'/>"
 "      </menu>"
 "    </menu>"
 "    <menu action='HelpMenu'>"
@@ -215,12 +233,12 @@ static void get_state_color (int Z, GdkColor *color, GChemTableApp *App)
 	App->GetStateColor (Z, color);
 }
 
-void on_changed_temp (GtkRange *range, GChemTableApp *app)
+static void on_changed_temp (GtkRange *range, GChemTableApp *app)
 {
 	app->SetTemperature (gtk_range_get_value (range));
 }
 
-void on_changed_family (GtkComboBox *box,  GChemTableApp *app)
+static void on_changed_family (GtkComboBox *box,  GChemTableApp *app)
 {
 	app->SetFamily (gtk_combo_box_get_active (box));
 }
@@ -230,15 +248,41 @@ static void get_family_color (int Z, GdkColor *color, GChemTableApp *App)
 	App->GetFamilyColor (Z, color);
 }
 
+static void get_acidity_color (int Z, GdkColor *color, GChemTableApp *App)
+{
+	App->GetAcidityColor (Z, color);
+}
+
+static void get_electroneg_color (int Z, GdkColor *color, GChemTableApp *App)
+{
+	App->GetElectronegColor (Z, color);
+}
+
+static void get_radius_color (int Z, GdkColor *color, GChemTableApp *App)
+{
+	App->GetRadiusColor (Z, color);
+}
+
+static void get_block_color (int Z, GdkColor *color, GChemTableApp *App)
+{
+	App->GetBlockColor (Z, color);
+}
+
+static void on_destroy (GChemTableApp *App)
+{
+	delete App;
+	gtk_main_quit ();
+}
+
 GChemTableApp::GChemTableApp (): Application ("gchemtable")
 {
 	GtkVBox* vbox;
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title (GTK_WINDOW(window), _("Periodic table of the elements"));
-	g_signal_connect (G_OBJECT (window), "destroy",
-		 G_CALLBACK (gtk_main_quit),
-		 NULL);
+	g_signal_connect_swapped (G_OBJECT (window), "delete-event",
+		 G_CALLBACK (on_destroy),
+		 this);
 
 	g_object_set (G_OBJECT(window), "allow-shrink", FALSE, NULL);
 	
@@ -299,10 +343,63 @@ GChemTableApp::GChemTableApp (): Application ("gchemtable")
 	gtk_combo_box_set_active (GTK_COMBO_BOX(familywidget), 0);
 	family = -1;
 	g_signal_connect (G_OBJECT (familywidget), "changed", G_CALLBACK (on_changed_family), this);
+
+	GladeXML *acidityxml = glade_xml_new (GLADEDIR"/acidity.glade", "acidity-legend", NULL);
+	GtkWidget *aciditylegend = glade_xml_get_widget (acidityxml, "acidity-legend");
+	colorschemes["acidity"] = gtk_periodic_add_color_scheme (periodic, (GtkPeriodicColorFunc) get_acidity_color, aciditylegend, this);
+	gtk_widget_show_all (aciditylegend);
+
+	colorschemes["electroneg"] = gtk_periodic_add_color_scheme (periodic, (GtkPeriodicColorFunc) get_electroneg_color, NULL, this);
+
+	colorschemes["radius"] = gtk_periodic_add_color_scheme (periodic, (GtkPeriodicColorFunc) get_radius_color, NULL, this);
+
+	GladeXML *blockxml = glade_xml_new (GLADEDIR"/block.glade", "block-legend", NULL);
+	GtkWidget *blocklegend = glade_xml_get_widget (blockxml, "block-legend");
+	colorschemes["block"] = gtk_periodic_add_color_scheme (periodic, (GtkPeriodicColorFunc) get_block_color, blocklegend, this);
+	gtk_widget_show_all (blocklegend);
+	gct_data_init ();
+
 }
 
 GChemTableApp::~GChemTableApp ()
 {
+	gct_data_clear ();
+}
+
+void GChemTableApp::OnAbout ()
+{
+	const gchar * authors[] = {"Jean Bréfort", NULL};
+	const gchar * comments = _("GChemTable is a chemical periodic table of the elements application");
+	/* const gchar * documentors[] = {NULL}; */
+	const gchar * copyright = _("Copyright © 2005-2007 Jean Bréfort");
+	const gchar * license =
+		"This program is free software; you can redistribute it and/or\n"
+		"modify it under the terms of the GNU General Public License as\n"
+		"published by the Free Software Foundation; either version 2 of the\n"
+		"License, or (at your option) any later version.\n\n"
+		"This program is distributed in the hope that it will be useful,\n"
+		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+		"GNU General Public License for more details.\n\n"
+		"You should have received a copy of the GNU General Public License\n"
+		"along with this program; if not, write to the Free Software\n"
+		"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307\n"
+		"USA";
+	
+	gtk_about_dialog_set_url_hook(on_about_activate_url, NULL, NULL);
+
+	/* Note to translators: replace the following string with the appropriate credits for you lang */
+	const gchar * translator_credits = _("translator_credits");
+	gtk_show_about_dialog (GetWindow (),
+	                       "name", "GChemTable",
+	                       "authors", authors,
+	                       "comments", comments,
+	                       "copyright", copyright,
+	                       "license", license,
+	                       "translator_credits", translator_credits,
+	                       "version", VERSION,
+	                       "website", "http://www.nongnu.org/gchemutils",
+	                       NULL);
 }
 
 void GChemTableApp::OnElement (int Z)
@@ -454,3 +551,125 @@ void GChemTableApp::GetFamilyColor (int Z, GdkColor *color)
 		return;
 	}
 }
+
+void GChemTableApp::GetAcidityColor (int Z, GdkColor *color)
+{
+	color->red= color->green = color->blue = 0;
+	Element *elt = Element::GetElement (Z);
+	int value = elt->GetIntegerProperty ("acidicbehaviour");
+	if (value == GCU_ERROR)
+		return;
+
+/*
+	0 means acidic
+	1 means basic
+	2 means neutral
+	3 means amphoteric
+*/
+
+	switch (value) {
+	case 0:
+		color->red = 0xffff;
+		return;
+
+	case 1:
+		color->blue = 0xffff;
+		return;
+
+	case 2:
+		color->green = 0xffff;
+		return;
+
+	case 3:
+		color->red = 0xffff;
+		color->blue = 0xffff;
+		return;
+	}
+}
+
+void GChemTableApp::GetElectronegColor (int Z, GdkColor *color)
+{
+	double max=3.98;
+	double min=0.7;
+	double limit;
+
+	color->red= color->green = color->blue = 0;
+	Element *elt = Element::GetElement (Z);
+	Value const *value = elt->GetProperty ("electronegativityPauling");
+	if (!value)
+		return;
+
+	double en = value->GetAsDouble ();
+
+	limit = 0.5 * (max - min);
+
+	if (en < limit) {
+		color->red = 0xffff;
+		color->blue = static_cast<guint16> ((en - min) * 0xffff / (limit - min));
+	} else {
+		color->blue = 0xffff;
+		color->red= static_cast<guint16> ((en - max) * 0xffff / (limit - max));
+	}
+
+}
+
+void GChemTableApp::GetRadiusColor (int Z, GdkColor *color)
+{
+	double max=2.25;
+	double min=0.32;
+	double limit;
+
+	color->red = color->green = color->blue = 0;
+	Element *elt = Element::GetElement (Z);
+	Value const *value = elt->GetProperty ("radiusCovalent");
+	if (!value)
+		return;
+
+	double radius = value->GetAsDouble ();
+
+	limit = 0.5 * (max - min);
+
+	if (radius < limit) {
+		color->red = 0xffff;
+		color->blue = static_cast<guint16> ((radius - min) * 0xffff / (limit - min));
+	} else {
+		color->blue = 0xffff;
+		color->red= static_cast<guint16> ((radius - max) * 0xffff / (limit - max));
+	}
+
+}
+
+void GChemTableApp::GetBlockColor (int Z, GdkColor *color)
+{
+	color->red= color->green = color->blue = 0;
+        Element *elt = Element::GetElement (Z);
+	std::string &value = elt->GetStringProperty ("periodTableBlock");
+	if (!value.length())
+		return;
+
+	if (value == "s") {
+		color->blue = 0x8eff;
+		return;
+	}
+
+	if (value == "p") {
+		color->red = 0x8eff;
+		return;
+	}
+
+	if (value == "d") {
+		color->green = 0x8eff;
+		return;
+	}
+
+	if (value == "f") {
+		color->blue = 0x8eff;
+		color->red = 0x8eff;
+		return;
+	}
+}
+
+void GChemTableApp::OnNewChart ()
+{
+	new GChemTableCurve (this, NULL);
+}
